class DocPicture:

    def init_attr(self, fig_num: int):
        self.width = 2.36 * fig_num

    def __init__(self, path_or_stream, fig_num: int = 1) -> None:
        self.path_or_stream = path_or_stream
        self.init_attr(fig_num)



from PIL import Image, ImageDraw
import warnings


class BasicPNGPlotter:
    def __init__(self, width, height, bg=(0, 0, 0, 0)):
        self.width = width
        self.height = height
        self.image = Image.new("RGBA", (width, height), bg)
        self.draw = ImageDraw.Draw(self.image)

    def draw_horizental_line(self, s_x, s_y, length, color="black", line_width=2):
        """画一段水平线，length为长度，正代表向右"""
        self.draw_line(s_x, s_y, s_x + length, s_y, color, line_width)

    def draw_vertical_line(self, s_x, s_y, length, color="black", line_width=2):
        """画一段竖直线，length为长度，正代表向下"""
        self.draw_line(s_x, s_y, s_x + length, s_y, color, line_width)

    def draw_rectangle(
        self, s_x, s_y, rec_width, rec_height, color="black", line_width=2
    ):
        """画一个矩形，左上角点为s_x,s_y，width正代表向右，height正代表向下"""
        self.draw_horizental_line(s_x, s_y, rec_width, color, line_width)
        self.draw_horizental_line(s_x, s_y + rec_height, rec_width, color, line_width)
        self.draw_vertical_line(s_x, s_y, rec_height, color, line_width)
        self.draw_vertical_line(s_x + rec_width, s_y, rec_height, color, line_width)

    def draw_line(self, s_x, s_y, e_x, e_y, color="black", width=2):
        if s_x < 0 or s_x > self.width or s_y < 0 or s_y > self.height:
            warnings.warn("Start point is out of figure.")
        if e_x < 0 or e_x > self.width or e_y < 0 or e_y > self.height:
            warnings.warn("End point is out of figure.")
        self.draw.line((s_x, s_y, e_x, e_y), fill=color, width=width)

    def draw_png(self):
        self.draw.line((0, 0, 500, 500), fill="black", width=2)

    def save(self, path):
        self.image.save(path, "PNG")


if __name__ == "__main__":
    p = BasicPNGPlotter(1500, 800)
    p.draw_png()
    p.save("testfiles/output.png")



import math
import numpy as np


class Component:

    def init_attr(self):
        self.E = 30000
        self.B = 1000
        self.H = 150
        self.vertical_q = 10
        self.length = math.sqrt(
            (self.end_point[1] - self.start_point[1]) ** 2
            + (self.end_point[0] - self.start_point[0]) ** 2
        )
        if self.start_point[0] == self.end_point[0]:
            self.alpha = 0
        else:
            self.alpha = math.atan(
                (self.end_point[1] - self.start_point[1])
                / (self.end_point[0] - self.start_point[0])
            )

    def __init__(self, p1, p2) -> None:
        self.start_point = p1
        self.end_point = p2
        self.section_kind = "Rectangle"
        self.init_attr()

    def set_comp_attr(self, e, b, h):
        self.E = e
        self.B = b
        self.H = h

    def set_vertical_q(self, q):
        self.vertical_q = q

    def create_k(self):
        self.calculate_section_attr()
        K = np.zeros((6, 6))

        l = self.length
        i = self.E * self.I / l
        B = self.E * self.area / l
        c_x = math.cos(self.alpha)
        c_y = math.sin(self.alpha)
        a1 = B * c_x * c_x + 12 * i / l / l * c_y * c_y
        a2 = (B - 12 * i / l / l) * c_x * c_y
        a3 = B * c_y * c_y + 12 * i / l / l * c_x * c_x
        a4 = -6 * i / l * c_y
        a5 = 6 * i / l * c_x
        a6 = 4 * i
        K[0, 0] = K[3, 3] = a1
        K[0, 3] = K[3, 0] = -a1
        K[1, 0] = K[4, 3] = K[0, 1] = K[3, 4] = a2
        K[1, 3] = K[0, 4] = K[3, 1] = K[4, 0] = -a2
        K[1, 1] = K[4, 4] = a3
        K[1, 4] = K[4, 1] = -a3
        K[0, 2] = K[0, 5] = K[2, 0] = K[5, 0] = a4
        K[2, 3] = K[3, 5] = K[3, 2] = K[5, 3] = -a4
        K[1, 2] = K[1, 5] = K[2, 1] = K[5, 1] = a5
        K[2, 4] = K[4, 5] = K[4, 2] = K[5, 4] = -a5
        K[2, 2] = K[5, 5] = a6
        K[2, 5] = K[5, 2] = a6 / 2
        return K

    def create_f(self):
        F = np.zeros((6, 1))
        # 这里只针对竖向荷载
        length = abs(self.end_point[0] - self.start_point[0])

        F[1, 0] = -self.vertical_q * length / 2
        F[2, 0] = -self.vertical_q * length**2 / 12
        F[4, 0] = -self.vertical_q * length / 2
        F[5, 0] = self.vertical_q * length**2 / 12
        return F

    def calculate_section_attr(self):
        if self.section_kind == "Rectangle":
            self.I = self.B * self.H**3 / 12
            self.area = self.B * self.H

    def set_f(self, f_1_x, f_1_y, f_2_x, f_2_y):
        self.f_1_x = f_1_x / 1e3
        self.f_1_y = f_1_y / 1e3
        self.f_2_x = f_2_x / 1e3
        self.f_2_y = f_2_y / 1e3

    def set_m(self, m_1, m_2):
        self.m_1 = m_1 / 1e6
        self.m_2 = m_2 / 1e6

    @property
    def v1(self):
        return self.f_1_y * np.cos(self.alpha) - self.f_1_x * np.sin(self.alpha)

    @property
    def v2(self):
        return -self.f_2_y * np.cos(self.alpha) + self.f_2_x * np.sin(self.alpha)



import math
from .StairComponent import Component
from typing import List


class Position:
    @property
    def lower_elevation(self):
        return min(self.left_elevation, self.right_elevation)

    @property
    def higher_elevation(self):
        return max(self.left_elevation, self.right_elevation)

    @property
    def total_height(self):
        return self.higher_elevation - self.lower_elevation

    @property
    def left_plat_length(self):
        return self.left_x2 - self.left_x1

    @property
    def right_plat_length(self):
        return self.right_x2 - self.right_x1

    @property
    def main_plat_length(self):
        return self.right_x1 - self.left_x2

    def __init__(
        self, left_elevation, right_elevation, left_x1, left_x2, right_x1, right_x2
    ):
        self.left_elevation = left_elevation
        self.right_elevation = right_elevation
        self.left_x1 = left_x1
        self.left_x2 = left_x2
        self.right_x1 = right_x1
        self.right_x2 = right_x2


class StairBeam:

    def __init__(self, width, height, offset):
        self.width = width
        self.height = height
        self.offset = offset


class StairPart:

    @property
    def stair_type(self):
        left_extend = self.beam_list[1].offset + self.beam_list[1].width / 2
        right_extend = self.beam_list[2].offset + self.beam_list[2].width / 2
        if left_extend >= self.STAIR_EXTEND_LIMIT:
            if right_extend >= self.STAIR_EXTEND_LIMIT:
                return "DT"
            else:
                return "BT"
        else:
            if right_extend >= self.STAIR_EXTEND_LIMIT:
                return "CT"
            else:
                return "AT"

    @property
    def stair_elevation_range(self):
        return f"{self.position.lower_elevation/1000:.3f}~{self.position.higher_elevation/1000:.3f}"

    @property
    def total_height(self):
        return abs(self.position.left_elevation - self.position.right_elevation)

    @property
    def equivlent_main_slab_thick(self):
        slope = math.atan(self.position.total_height / self.position.main_plat_length)
        thick_1 = self.main_thick / math.cos(slope)
        thick_2 = self.position.total_height / self.step_num / 2
        return thick_1 + thick_2

    @property
    def total_horizental_length(self):
        length = self.position.right_x1 - self.position.left_x2
        length += self.beam_list[1].offset + self.beam_list[1].width / 2
        length += self.beam_list[2].offset + self.beam_list[2].width / 2
        return length

    @property
    def left_extend_length(self):
        if self.stair_type == "AT" or self.stair_type == "CT":
            return 0
        else:
            return self.beam_list[1].offset + self.beam_list[1].width / 2

    @property
    def right_extend_length(self):
        if self.stair_type == "AT" or self.stair_type == "BT":
            return 0
        else:
            return self.beam_list[2].offset + self.beam_list[2].width / 2

    @property
    def stair_length_list(self):
        return [
            self.left_extend_length,
            self.total_horizental_length
            - self.left_extend_length
            - self.right_extend_length,
            self.right_extend_length,
        ]

    @property
    def up_real_rebar_str(self):
        return f"E{self.up_d:d}@{self.up_dis:d}"

    @property
    def up_real_rebar_area(self):
        return f"{(self.up_d*self.up_d * math.pi/4/self.up_dis):.3f}"

    def get_calculate_moments(self) -> List[float]:
        if self.stair_type == "AT":
            return [0, 0, self.components[0].m_2, 0, 0]
        elif self.stair_type == "BT":
            return [0, self.components[0].m_2, self.components[1].m_2, 0, 0]
        elif self.stair_type == "CT":
            return [0, 0, self.components[0].m_2, self.components[1].m_2, 0]
        elif self.stair_type == "DT":
            return [
                0,
                self.components[0].m_2,
                self.components[1].m_2,
                self.components[2].m_2,
                0,
            ]
        return [0, 10, 20, 10, 0]

    def get_calculate_shears(self) -> List[float]:
        if self.stair_type == "AT":
            return [
                0,
                0,
                self.components[0].v1,
                self.components[0].v2,
                self.components[1].v1,
                self.components[1].v2,
                0,
                0,
            ]
        elif self.stair_type == "BT":
            return [
                self.components[0].v1,
                self.components[0].v2,
                self.components[1].v1,
                self.components[1].v2,
                self.components[2].v1,
                self.components[2].v2,
                0,
                0,
            ]
        elif self.stair_type == "CT":
            return [
                0,
                0,
                self.components[0].v1,
                self.components[0].v2,
                self.components[1].v1,
                self.components[1].v2,
                self.components[2].v1,
                self.components[2].v2,
            ]
        elif self.stair_type == "DT":
            return [
                self.components[0].v1,
                self.components[0].v2,
                self.components[1].v1,
                self.components[1].v2,
                self.components[2].v1,
                self.components[2].v2,
                self.components[3].v1,
                self.components[3].v2,
            ]
        return [0, 0, 0, 10, 20, 10, 0, 0]

    def get_left_slab_table_moments(self):
        moments = self.get_calculate_moments()
        return [0, moments[1] * 0.25, moments[1]]

    def get_main_table_moments(self):
        moments = self.get_calculate_moments()
        return [moments[1], moments[2], moments[3]]

    def get_right_slab_table_moments(self):
        moments = self.get_calculate_moments()
        return [moments[3], moments[3] * 0.25, 0]

    def get_left_slab_table_shears(self):
        shears = self.get_calculate_shears()
        return [shears[0], (shears[0] + shears[1]) / 2, shears[1]]

    def get_main_table_shears(self):
        shears = self.get_calculate_shears()
        return [shears[2], (shears[2] + shears[5]) / 2, shears[5]]

    def get_right_slab_table_shears(self):
        shears = self.get_calculate_shears()
        return [shears[6], (shears[6] + shears[7]) / 2, shears[7]]

    def get_shear_validate(self, which_side, ft, cover_thick):
        if which_side == "left":
            shears = self.get_left_slab_table_shears()
            shear_limit = 0.7 * 1 * ft * (self.left_thick - cover_thick) * 1000 / 1000
        elif which_side == "right":
            shears = self.get_right_slab_table_shears()
            shear_limit = 0.7 * 1 * ft * (self.right_thick - cover_thick) * 1000 / 1000
        else:
            shears = self.get_main_table_shears()
            shear_limit = 0.7 * 1 * ft * (self.main_thick - cover_thick) * 1000 / 1000
        max_shear = max([abs(i) for i in shears])
        if max_shear <= shear_limit:
            shear_context = f"Vmax={max_shear:.2f}kN < 0.7βhftbh0={shear_limit:.2f}kN，抗剪截面满足要求！"
        else:
            shear_context = f"Vmax={max_shear:.2f}kN > 0.7βhftbh0={shear_limit:.2f}kN，抗剪截面不满足要求！"

        return shear_context

    def init_default_data(self):
        self.stair_width = 1500
        self.stair_well_width = 100
        self.beam_list = [
            StairBeam(300, 500, 0),
            StairBeam(300, 500, 0),
            StairBeam(300, 500, 0),
            StairBeam(300, 500, 0),
        ]
        self.set_thickness(140, 140, 140)
        self.set_real_rebar(10, 150, 10, 150)

    def __init__(self, position: Position, step_num):
        self.position = position
        self.step_num = step_num
        self.init_default_data()

        self.STAIR_EXTEND_LIMIT = 200

    def set_thickness(self, left_thick, main_thick, right_thick):
        self.left_thick = left_thick
        self.main_thick = main_thick
        self.right_thick = right_thick

    def set_beam_offset(self, i, offset):
        self.beam_list[i].offset = offset

    def set_real_rebar(self, up_d, up_dis, down_d, down_dis):
        self.up_d = up_d
        self.up_dis = up_dis
        self.down_d = down_d
        self.down_dis = down_dis

    def set_calculate_result(self, components: List[Component]):
        self.components = components



import warnings
from enum import Enum
from docx import Document
from docx.document import Document as doc
from docx.shared import Inches, RGBColor, Pt, Cm
from docx.oxml.ns import qn, nsdecls
from docx.enum.text import WD_PARAGRAPH_ALIGNMENT, WD_BREAK
from docx.enum.table import WD_CELL_VERTICAL_ALIGNMENT, WD_TABLE_ALIGNMENT
from docx.oxml.parser import parse_xml
from docx.oxml.xmlchemy import BaseOxmlElement
from docx.enum.section import WD_ORIENTATION
from .DocParagraph import DocParagraph
from .DocPicture import DocPicture
from .DocTable import DocTable
from .UtilFunctions import set_cell_border, analysis_sub_and_super_script

FONT_STR = "w:eastAsia"


class Page:
    def __init__(self, width: float, height: float, is_landscape=False):
        self._width = width
        self._height = height
        self.is_landscape = is_landscape

    @property
    def width(self):
        return self._height if self.is_landscape else self._width

    @property
    def height(self):
        return self._width if self.is_landscape else self._height


class PageSize(Enum):
    A4 = Page(210, 297)
    A4_LANDSCAPE = Page(210, 297, True)
    A3 = Page(297, 420)
    A3_LANDSCAPE = Page(297, 420, True)


class BasicGenerator:

    def __init__(self):
        self.mm_factor = 36000
        self.doc = Document()
        self.doc: doc
        self.init_styles()
        self.highlighted_color = 7

    def init_styles(self):
        styles = self.doc.styles
        self.body_style = self.init_body_style(styles)
        self.table_style = self.init_table_style(styles)
        self.small_title_style = self.init_small_title_style(styles)

    def init_body_style(self, styles):
        """生成一个用于普通段落文字的body_style

        Args:
            styles (_type_): _description_

        Returns:
            _type_: _description_
        """
        body_style = styles["Body Text"]
        body_style.font.name = "Times New Roman"
        body_style.paragraph_format.first_line_indent = Inches(0.32)
        body_style.paragraph_format.space_before = Pt(0)
        body_style.paragraph_format.space_after = Pt(0)
        body_style._element.rPr.rFonts.set(qn(FONT_STR), "宋体")
        return body_style

    def init_table_style(self, styles):
        """生成一个用于表格内部段落文字的table_style

        Args:
            styles (_type_): _description_

        Returns:
            _type_: _description_
        """
        table_style = styles["Normal"]
        table_style.font.name = "Time New Roman"
        table_style.font.size = Pt(10)
        table_style._element.rPr.rFonts.set(qn(FONT_STR), "宋体")
        table_style.paragraph_format.space_before = Pt(0)
        table_style.paragraph_format.space_after = Pt(0)
        return table_style

    def init_small_title_style(self, styles):
        """生成用于表格名称和图片名称的small_title_style

        Args:
            styles (_type_): _description_

        Returns:
            _type_: _description_
        """
        small_title_style = styles["Body Text 2"]
        small_title_style.font.name = "Times New Roman"
        small_title_style._element.rPr.rFonts.set(qn(FONT_STR), "黑体")
        small_title_style.paragraph_format.space_before = Pt(6)
        small_title_style.paragraph_format.space_after = Pt(3)
        small_title_style.paragraph_format.line_spacing = Pt(15)
        return small_title_style

    def change_paper_size(self, page_size: PageSize, column_num: int = 1):
        section = self.doc.sections[0]
        section.page_width = int(page_size.value.width * self.mm_factor)
        section.page_heigth = int(page_size.value.height * self.mm_factor)
        if page_size.value.is_landscape:
            section.orientation = WD_ORIENTATION.LANDSCAPE
        sect_pr = section._sectPr
        cols = sect_pr.xpath("./w:cols")[0]
        cols.set(qn("w:num"), str(column_num))

    def change_paper_margin(self, left: int, top: int, right: int, bottom: int):
        """改变页边距，单位为mm，按照左、上、右、下的顺序"""
        section = self.doc.sections[0]
        section.top_margin = int(self.mm_factor * top)
        section.bottom_margin = int(self.mm_factor * bottom)
        section.left_margin = int(self.mm_factor * left)
        section.right_margin = int(self.mm_factor * right)

    def add_big_title(self, context):
        p = self.doc.add_paragraph(context)
        run = p.runs[0]
        run.font.size = Pt(14)
        run.font.bold = True
        p.alignment = WD_PARAGRAPH_ALIGNMENT.CENTER

    def add_title(
        self, title: DocParagraph, space_before: float = 0, space_after: float = 0
    ):
        """为了添加大纲等级而添加的函数，不需要大纲等级时不要使用

        Args:
            title (DocParagraph): _description_
        """
        p = self.doc.add_heading("", level=title.par_level)
        run = p.add_run(title.context)
        run.font.name = "Times New Roman"
        run._element.rPr.rFonts.set(qn("w:eastAsia"), "宋体")
        run.font.color.rgb = RGBColor(0, 0, 0)
        par_format = p.paragraph_format
        par_format.space_before = Pt(space_before)
        par_format.space_after = Pt(space_after)

        self.paragraph_format_double_check(p, title)

    def add_paragraph(self, par: DocParagraph):
        if par.context == None or par.context == "":
            warnings.warn("This par has no context.")
            return
        if "{" in par.context:
            a, b = analysis_sub_and_super_script(par.context)
            if par.style != None:
                p = self.doc.add_paragraph("", par.style)
            else:
                p = self.doc.add_paragraph()
            self.add_context_with_sub_or_super(p, a, b)
        elif par.style != None:
            p = self.doc.add_paragraph(par.context, par.style)
        else:
            p = self.doc.add_paragraph(par.context)
        self.paragraph_format_double_check(p, par)

    def add_context_with_sub_or_super(self, p, str_list, sub_or_super):
        for i in range(len(str_list)):
            run = p.add_run(str_list[i])
            run.font.name = "Times New Roman"
            if sub_or_super[i] == 1:
                run.font.subscript = True
            elif sub_or_super[i] == 2:
                run.font.superscript = True
            elif sub_or_super[i] == 3:
                run.font.highlight_color = self.highlighted_color
            run.font.size = Pt(11)

    def paragraph_format_double_check(self, p, doc_par: DocParagraph):
        run = p.runs[0]
        if doc_par.font_size != None:
            run.font.size = Pt(doc_par.font_size)
        if doc_par.is_bold != None:
            run.font.bold = doc_par.is_bold
        par_format = p.paragraph_format
        if doc_par.first_line_indent != None:
            par_format.first_line_indent = Inches(doc_par.first_line_indent)
        if doc_par.alignment != None:
            par_format.alignment = doc_par.alignment
        if doc_par.font_size != None:
            run.font.size = Pt(doc_par.font_size)

    def add_table(self, my_table: DocTable):
        if my_table.title != None:
            p = self.doc.add_paragraph(my_table.title, self.small_title_style)
            p.alignment = WD_PARAGRAPH_ALIGNMENT.CENTER
        table = self.doc.add_table(
            rows=my_table.row_num, cols=my_table.column_num, style="Table Grid"
        )
        table.alignment = WD_TABLE_ALIGNMENT.CENTER
        for i, j, k, p in my_table.merged_cells:
            table.cell(i, j).merge(table.cell(k, p))
        for i in range(my_table.row_num):
            for j in range(my_table.column_num):
                self.add_table_cell(table, my_table, i, j)
        self.tab_bg_color(table, my_table.row_num, my_table.column_num)

    def tab_bg_color(self, table, rows, cols, color_str="CCCCCC"):
        """表格样式调整"""
        shading_list = locals()
        for i in range(cols):
            shading_list["shading_elm_" + str(i)] = parse_xml(
                r'<w:shd {} w:fill="{bgColor}"/>'.format(
                    nsdecls("w"), bgColor=color_str
                )
            )
            table.rows[0].cells[i]._tc.get_or_add_tcPr().append(
                shading_list["shading_elm_" + str(i)]
            )
        for i in range(rows):
            shading_list["shading_elm_" + str(cols + i)] = parse_xml(
                r'<w:shd {} w:fill="{bgColor}"/>'.format(
                    nsdecls("w"), bgColor=color_str
                )
            )
            table.rows[i].cells[0]._tc.get_or_add_tcPr().append(
                shading_list["shading_elm_" + str(cols + i)]
            )
        table_cells = table._cells
        for i in range(rows):
            table_row = table_cells[i * cols : i * cols + cols]
            for k in range(cols):
                table_row[k].vertical_alignment = WD_CELL_VERTICAL_ALIGNMENT.CENTER
                table_row[k].paragraphs[
                    0
                ].paragraph_format.alignment = WD_TABLE_ALIGNMENT.CENTER
                if i == 0:
                    runs = table_row[k].paragraphs[0].runs
                    for m in range(len(runs)):
                        runs[m].bold = True
                table.rows[i].height = Cm(0.7)

    def add_table_cell(self, table, my_table: DocTable, i, j):
        try:
            cell = table.cell(i, j)
            _ = my_table.context[i][j]
        except IndexError:
            return
        if "{" in my_table.context[i][j]:
            a, b = analysis_sub_and_super_script(my_table.context[i][j])
            p = cell.paragraphs[0]
            self.add_context_with_sub_or_super(p, a, b)
        else:
            cell.text = my_table.context[i][j]
            cell.paragraphs[0].style = self.table_style
            p = cell.paragraphs[0]
        p.alignment = WD_PARAGRAPH_ALIGNMENT.CENTER
        if my_table.no_grid:
            set_cell_border(
                cell,
                top={"color": "#FFFFFF"},
                bottom={"color": "#FFFFFF"},
                start={"color": "#FFFFFF"},
                end={"color": "#FFFFFF"},
            )
        if my_table.all_bold:
            p.runs[0].font.bold = True

    def add_picture(self, doc_picture: DocPicture):
        self.doc.add_picture(
            doc_picture.path_or_stream, width=Inches(doc_picture.width)
        )
        last_paragraph = self.doc.paragraphs[-1]
        last_paragraph.alignment = 1

    def add_blank_paragraph(self):
        self.doc.add_paragraph()

    def add_page_break(self):
        if len(self.doc.paragraphs[-1].runs) > 0:
            self.doc.paragraphs[-1].runs[0].add_break(WD_BREAK.PAGE)
        else:
            run = self.doc.paragraphs[-1].add_run("")
            run.add_break(WD_BREAK.PAGE)

    def save(self, path: str):
        for _ in range(10):
            try:
                self.doc.save(path)
            except Exception:
                path = path.replace(".docx", "1.docx")



class SteelLevel:
    def __init__(
        self,
        name: str,
        fy: float,
        elastic_module: float,
    ):
        self.name = name
        self.fy = fy
        self.elastic_module = elastic_module


class Steel:
    HRB300 = SteelLevel("HRB300", 300, 2000000)
    HRB355 = SteelLevel("HRB355", 300, 2000000)
    HRB400 = SteelLevel("HRB400", 300, 2000000)
    HRB500 = SteelLevel("HRB500", 300, 2000000)
    Q235 = SteelLevel("Q235", 111, 2000000)
    Q345 = SteelLevel("Q345", 111, 2000000)
    Q355 = SteelLevel("Q355", 111, 2000000)
    Q420 = SteelLevel("Q420", 111, 2000000)



class DocParagraph:
    def init_attr(self):
        self.style = None
        self.font_size = 11
        self.first_line_indent = None
        self.space_before = None
        self.space_after = None
        self.line_spacing = None
        self.is_bold = None
        self.alignment = None
        self.par_level = None

    def __init__(self, context):
        self.context = context
        self.init_attr()



from .BasicGenerator import BasicGenerator, PageSize
from docx.shared import Inches, RGBColor, Pt, Cm
from CivilTools.YDBLoader.BuildingDefine.StairPart import (
    Position,
    StairPart,
    LoadParams,
    StairLoad,
    LoadCalulateType,
)
from .UtilFunctions import MatrixSolver
from .DocTable import DocTable
from .DocPicture import DocPicture
from .DocParagraph import DocParagraph


class StairCalculationReport(BasicGenerator):
    def __init__(self):
        super().__init__()
        # 修改为A3图纸，横向，两栏
        self.change_paper_size(PageSize.A4)
        # 修改纸张Margin，单位mm
        self.change_paper_margin(20, 20, 20, 20)
        # 格式统一修改
        # self.body_style.paragraph_format.line_spacing = Pt(22)
        self.set_const()

    def set_const(self):
        self.big_title = "现浇板式普通楼梯设计"
        self.operate_code_title = "执行规范："
        self.operate_code_text_list = [
            "《混凝土结构通用规范》(GB 55008-2021)，本文简称《混凝土通用规范》；",
            "《混凝土结构设计规范》(GB 50010-2010(2015年版))，本文简称《混凝土规范》；",
            "《工程结构通用规范》(GB 55001-2021)；",
            "《建筑结构荷载规范》(GB 50009-2012)，本文简称《荷载规范》；",
        ]
        self.symbol_explain = "钢筋：d-HPB300；D-HRB335；E-HRB400；F-RRB400；G-HRB500；Q-HRBF400；R-HRB550"
        self.cut_line = "---------------------------------------------------------------------------------------------"
        self.concrete_data = {
            30: [14.3, 1.43, 2.01, 30000],  # // fc, ft, ftk, E
            35: [16.7, 1.57, 2.20, 31500],  # // fc, ft, ftk, E
            40: [19.1, 1.71, 2.39, 32500],  # // fc, ft, ftk, E
        }
        self.concrete_density = 25
        self.rebar_level = "HRB400"
        self.rebar_fy = 360
        self.indent = 0.22
        self.normal_font_size = 11

    # 数据入口！！！！
    def set_stair_data(self):
        """这个函数需要重写！！！！！是数据流入的最主要的入口"""
        position1 = Position(0, 2180, 0, 1910, 5030, 6850)
        sp1 = StairPart(position1, 13)
        sp1.left_thick = sp1.main_thick = sp1.right_thick = 130
        position2 = Position(0, 2180, 0, 1910, 5030, 6850)
        sp2 = StairPart(position2, 13)
        sp2.set_beam_offset(1, 500)
        position3 = Position(0, 2180, 0, 1910, 5030, 6850)
        sp3 = StairPart(position3, 13)
        sp3.set_beam_offset(2, 500)
        position4 = Position(0, 2180, 0, 1910, 5030, 6850)
        sp4 = StairPart(position4, 13)
        sp4.set_beam_offset(1, 500)
        sp4.set_beam_offset(2, 500)

        self.stair_list = [sp1, sp2, sp3, sp4]

    # 数据入口！！！！
    def set_calculate_info(self):
        self.load_param = LoadParams()
        self.concrete_level = 30
        self.rebar_area_adjust_coef = 1
        self.cover_thickness = 15

        self.displacement_limit = 200
        self.crack_width_limit = 0.3

    def load_calculate(self):
        left_dead = (
            self.current_stair.left_thick * self.concrete_density / 1000
            + self.load_param.append_dead_load
        )
        self.left_slab_load = StairLoad(
            left_dead, self.load_param.live_load, self.load_param
        )

        right_dead = (
            self.current_stair.right_thick * self.concrete_density / 1000
            + self.load_param.append_dead_load
        )
        self.right_slab_load = StairLoad(
            right_dead, self.load_param.live_load, self.load_param
        )

        main_dead = (
            self.current_stair.equivlent_main_slab_thick * self.concrete_density / 1000
            + self.load_param.append_dead_load
        )
        self.main_slab_load = StairLoad(
            main_dead, self.load_param.live_load, self.load_param
        )

    def do_the_math(self):
        solver = MatrixSolver(self.current_stair)
        solver.set_load(
            self.left_slab_load.q, self.main_slab_load.q, self.right_slab_load.q
        )
        self.current_stair.set_calculate_result(solver.submit_problem())

    def add_first_part(self):
        # 添加最大的标题
        self.add_big_title(self.big_title)
        # 添加一个2*6的表格，包含项目基本信息和设计人员信息
        table = DocTable(2, 6)
        table_context = [
            ["项目名称", "XX项目", "构件编号", "XXXX", "日    期", "2024/10/10"],
            ["设    计", "", "校    对", "", "审    核", ""],
        ]
        table.set_table_context(table_context)

        # table.no_grid = True
        table.all_bold = True
        self.add_table(table)
        # 添加执行规范和符号示意
        doc_par = DocParagraph(self.operate_code_title)
        doc_par.style = self.body_style
        doc_par.is_bold = True
        doc_par.font_size = 10
        doc_par.first_line_indent = 0
        self.add_paragraph(doc_par)

        for line in self.operate_code_text_list:
            doc_par = DocParagraph(line)
            doc_par.style = self.body_style
            doc_par.font_size = 10
            doc_par.first_line_indent = 0.22
            self.add_paragraph(doc_par)
        self.add_blank_paragraph()
        doc_par = DocParagraph(self.symbol_explain)
        doc_par.style = self.body_style
        doc_par.font_size = 10
        doc_par.first_line_indent = 0
        self.add_paragraph(doc_par)
        doc_par.context = self.cut_line
        self.add_paragraph(doc_par)

    def add_single_stair(self, stair: StairPart, index):
        self.current_stair = stair
        self.current_index = index

        # 添加一个标题
        doc_par = DocParagraph(
            f"{self.current_index}、{self.current_stair.stair_type}类梯板，标高区间：{self.current_stair.stair_elevation_range}"
        )
        doc_par.style = self.body_style
        doc_par.first_line_indent = 0
        doc_par.font_size = 11
        doc_par.is_bold = True
        doc_par.par_level = 1
        self.add_title(doc_par)

        # 进行必要的计算
        self.load_calculate()

        self.do_the_math()

        # 添加part1已知条件
        self.add_1_basic_info()
        self.add_2_load_and_calculate()
        self.add_3_result()
        self.add_blank_paragraph()

    def add_1_basic_info(self):
        stair = self.current_stair
        index = self.current_index
        indent = self.indent
        font_size = self.normal_font_size
        # 添加一个二级标题
        doc_par = DocParagraph(f"{index}.1 已知条件")
        doc_par.style = self.body_style
        doc_par.first_line_indent = 0
        doc_par.font_size = font_size
        doc_par.is_bold = False
        doc_par.par_level = 2
        self.add_title(doc_par)

        # 几何信息
        doc_par = DocParagraph("几何信息：")
        doc_par.style = self.body_style
        doc_par.first_line_indent = indent
        doc_par.font_size = font_size
        self.add_paragraph(doc_par)

        doc_par = DocParagraph(
            f"左标高={stair.position.left_elevation/1000:.3f}m；右标高={stair.position.right_elevation/1000:.3f}m"
        )
        doc_par.style = self.body_style
        doc_par.first_line_indent = indent * 2
        doc_par.font_size = font_size
        self.add_paragraph(doc_par)
        doc_par.context = f"左平台长度={stair.position.left_plat_length}mm；右平台长度={stair.position.right_plat_length}mm"
        self.add_paragraph(doc_par)
        doc_par.context = (
            f"左平台厚度={stair.left_thick}mm；右平台厚度={stair.right_thick}mm"
        )
        self.add_paragraph(doc_par)
        doc_par.context = f"内延长（左）={stair.beam_list[1].offset}mm；内延长（右）={stair.beam_list[2].offset}mm"
        self.add_paragraph(doc_par)
        doc_par.context = f"梯段长度={stair.position.right_x1 - stair.position.left_x2}mm；踏步数={stair.step_num}"
        doc_par.is_bold = True
        self.add_paragraph(doc_par)
        doc_par.context = f"梯板厚度={stair.main_thick}mm；梯段宽度={stair.stair_width}mm；梯井宽度={stair.stair_well_width}mm"
        self.add_paragraph(doc_par)
        doc_par.context = "平面类型：单跑"
        doc_par.is_bold = False
        self.add_paragraph(doc_par)
        self.add_blank_paragraph()

        # 荷载信息
        doc_par = DocParagraph("荷载信息：")
        doc_par.style = self.body_style
        doc_par.first_line_indent = indent
        doc_par.font_size = font_size
        self.add_paragraph(doc_par)

        doc_par = DocParagraph(
            f"附加恒荷载={self.load_param.append_dead_load:.3f}kN/m^{{2}} 活荷载={self.load_param.live_load:.3f}kN/m^{{2}}"
        )
        doc_par.style = self.body_style
        doc_par.first_line_indent = indent * 2
        doc_par.font_size = font_size
        self.add_paragraph(doc_par)
        doc_par.context = f"恒载分项系数：{self.load_param.dead_load_coef}；活载分项系数：{self.load_param.live_load_coef}；活载调整系数：γ_{{L}} ={self.load_param.live_load_adjust_coef:.2f}"
        self.add_paragraph(doc_par)
        doc_par.context = (
            f"活载准永久值系数：{self.load_param.live_load_permenent_coef}"
        )
        self.add_paragraph(doc_par)
        doc_par.context = f"混凝土等级：C{self.concrete_level}，f_{{c}} ={self.concrete_data[self.concrete_level][0]:.2f}MPa"
        self.add_paragraph(doc_par)
        doc_par.context = f"混凝土容重：{self.concrete_density:.2f}kN/mm^{{3}}"
        self.add_paragraph(doc_par)
        doc_par.context = f"配筋调整系数：{self.rebar_area_adjust_coef:.2f}；纵筋保护层厚度：c={self.cover_thickness}mm"
        self.add_paragraph(doc_par)
        doc_par.context = (
            f"梯板纵筋等级：{self.rebar_level}；f_{{y}} ={self.rebar_fy}MPa"
        )
        self.add_paragraph(doc_par)
        doc_par.context = (
            f"梯梁纵筋等级：{self.rebar_level}；f_{{y}} ={self.rebar_fy}MPa"
        )
        self.add_paragraph(doc_par)
        doc_par.context = (
            f"梯梁箍筋等级：{self.rebar_level}；f_{{y}} ={self.rebar_fy}MPa"
        )
        self.add_paragraph(doc_par)

        # 验算信息
        doc_par = DocParagraph("验算信息：")
        doc_par.style = self.body_style
        doc_par.first_line_indent = indent
        doc_par.font_size = font_size
        self.add_paragraph(doc_par)

        doc_par = DocParagraph(
            f"挠度限值：L_{{0}} /{self.displacement_limit}；裂缝限值：{self.crack_width_limit}mm"
        )
        doc_par.style = self.body_style
        doc_par.first_line_indent = indent * 2
        doc_par.font_size = font_size
        self.add_paragraph(doc_par)

        # 计算要求
        doc_par = DocParagraph("计算要求：")
        doc_par.style = self.body_style
        doc_par.first_line_indent = indent
        doc_par.font_size = font_size
        self.add_paragraph(doc_par)

        doc_par = DocParagraph(
            "(1)楼梯板计算；(2)平台梁计算；(3)平台板裂缝验算；(4)平台板挠度验算；"
        )
        doc_par.style = self.body_style
        doc_par.first_line_indent = indent * 2
        doc_par.font_size = font_size
        self.add_paragraph(doc_par)

    def add_2_load_and_calculate(self):
        index = self.current_index
        indent = self.indent
        font_size = self.normal_font_size
        # 添加一个二级标题
        doc_par = DocParagraph(f"{index}.2 荷载与内力计算")
        doc_par.style = self.body_style
        doc_par.first_line_indent = 0
        doc_par.font_size = font_size
        doc_par.is_bold = False
        doc_par.par_level = 2
        self.add_title(doc_par)

        doc_par = DocParagraph("(1)荷载计算：")
        doc_par.style = self.body_style
        doc_par.first_line_indent = 0
        doc_par.font_size = font_size
        self.add_paragraph(doc_par)
        # 标准值
        doc_par.context = "标准值(qk)："
        doc_par.first_line_indent = indent
        self.add_paragraph(doc_par)
        doc_par.context = self.get_load_text(LoadCalulateType.qk)
        doc_par.first_line_indent = indent * 2
        self.add_paragraph(doc_par)
        # 设计值
        doc_par.context = "设计值(q)："
        doc_par.first_line_indent = indent
        self.add_paragraph(doc_par)
        doc_par.context = self.get_load_text(LoadCalulateType.q)
        doc_par.first_line_indent = indent * 2
        self.add_paragraph(doc_par)
        # 准永久值
        doc_par.context = "准永久值(qe)："
        doc_par.first_line_indent = indent
        self.add_paragraph(doc_par)
        doc_par.context = self.get_load_text(LoadCalulateType.qe)
        doc_par.first_line_indent = indent * 2
        self.add_paragraph(doc_par)

        doc_par.context = "(2)内力计算："
        doc_par.first_line_indent = 0
        self.add_paragraph(doc_par)
        # 标准值
        doc_par.context = "a.楼梯板：矩阵位移法求解。"
        doc_par.first_line_indent = indent
        self.add_paragraph(doc_par)

        self.add_blank_paragraph()

    def get_load_text(self, type: LoadCalulateType):
        if type == LoadCalulateType.qk:
            Q1 = self.left_slab_load.qk
            Q2 = self.main_slab_load.qk
            Q3 = self.right_slab_load.qk
        elif type == LoadCalulateType.q:
            Q1 = self.left_slab_load.q
            Q2 = self.main_slab_load.q
            Q3 = self.right_slab_load.q
        elif type == LoadCalulateType.qe:
            Q1 = self.left_slab_load.qe
            Q2 = self.main_slab_load.qe
            Q3 = self.right_slab_load.qe
        else:
            Q1 = Q2 = Q3 = 0

        if self.current_stair.stair_type == "AT":
            return f"斜梯段：{Q2:.2f}kN/m"
        elif self.current_stair.stair_type == "BT":
            return f"左延伸段：{Q1:.2f}kN/m\t斜梯段：{Q2:.2f}kN/m"
        elif self.current_stair.stair_type == "CT":
            return f"斜梯段：{Q2:.2f}kN/m\t右延伸段：{Q3:.2f}kN/m"
        elif self.current_stair.stair_type == "DT":
            return (
                f"左延伸段：{Q1:.2f}kN/m\t斜梯段：{Q2:.2f}kN/m\t右延伸段：{Q3:.2f}kN/m"
            )
        else:
            return "未知踏步类型，暂无数据"

    def add_3_result(self):
        index = self.current_index
        indent = self.indent
        font_size = self.normal_font_size
        # 添加一个二级标题
        doc_par = DocParagraph(f"{index}.3 计算结果")
        doc_par.style = self.body_style
        doc_par.first_line_indent = 0
        doc_par.font_size = font_size
        doc_par.is_bold = False
        doc_par.par_level = 2
        self.add_title(doc_par)

        doc_par = DocParagraph("计算说明：")
        doc_par.style = self.body_style
        doc_par.first_line_indent = 0
        doc_par.font_size = font_size
        self.add_paragraph(doc_par)

        doc_par.context = "(a)简化方法：取板沿着宽度方向单位长度的板带"
        doc_par.first_line_indent = indent
        self.add_paragraph(doc_par)

        doc_par.context = "(b)计算方法：矩阵位移法"
        self.add_paragraph(doc_par)

        doc_par.context = "单位说明："
        doc_par.first_line_indent = 0
        self.add_paragraph(doc_par)

        doc_par.context = "弯        矩：kN·m\t\t剪        力：kN/m\t\t挠        度：mm"
        doc_par.first_line_indent = indent
        self.add_paragraph(doc_par)
        doc_par.context = r"纵筋面积：mm^{2}/m\t\t截面尺寸：mm×mm\t\t裂        缝：mm"
        self.add_paragraph(doc_par)

        self.add_blank_paragraph()
        doc_par.context = "板段配筋计算结果："
        doc_par.first_line_indent = 0
        self.add_paragraph(doc_par)
        doc_par.context = "---------------------------------------------------------------------------------------------"
        self.add_paragraph(doc_par)

        self.insert_calculate_table()

        # self.insert_all_pictures()

        self.add_blank_paragraph()

    def insert_calculate_table(self):
        table_index = 1
        ft = self.concrete_data[self.concrete_level][1]
        if (
            self.current_stair.stair_type == "BT"
            or self.current_stair.stair_type == "DT"
        ):
            left_table = DocTable(8, 4)
            left_table.merge_cells(3, 1, 3, 3)
            left_table.set_table_title(
                f"计算板段-{table_index}(左延伸段)：截面B×H=1000mm×{self.current_stair.left_thick}mm"
            )
            moments = self.current_stair.get_left_slab_table_moments()
            shears = self.current_stair.get_left_slab_table_shears()
            shear_context = self.current_stair.get_shear_validate(
                "left", ft, self.cover_thickness
            )

            table_context = [
                ["截面位置", "左", "中", "右"],
                [
                    "弯矩(M)",
                    f"{moments[0]:.2f}",
                    f"{moments[1]:.2f}",
                    f"{moments[2]:.2f}",
                ],
                ["剪力(V)", f"{shears[0]:.2f}", f"{shears[1]:.2f}", f"{shears[2]:.2f}"],
                ["抗剪截面验算", shear_context],
                ["上部计算纵筋As'", "", "", ""],
                ["下部计算纵筋As", "", "", ""],
                ["上部纵筋实配", "", "", ""],
                ["下部纵筋实配", "", "", ""],
            ]
            left_table.set_table_context(table_context)
            self.add_table(left_table)
            table_index += 1

        cal_table = DocTable(13, 4)
        for i in [3, 8, 9, 11, 12]:
            cal_table.merge_cells(i, 1, i, 3)
        cal_table.set_table_title(
            f"计算板段-{table_index}(斜梯段)：截面B×H=1000mm×{self.current_stair.main_thick}mm"
        )
        moments = self.current_stair.get_main_table_moments()
        shears = self.current_stair.get_main_table_shears()
        shear_context = self.current_stair.get_shear_validate(
            "main", ft, self.cover_thickness
        )

        table_context = [
            ["截面位置", "左", "中", "右"],
            ["弯矩(M)", f"{moments[0]:.2f}", f"{moments[1]:.2f}", f"{moments[2]:.2f}"],
            ["剪力(V)", f"{shears[0]:.2f}", f"{shears[1]:.2f}", f"{shears[2]:.2f}"],
            ["抗剪截面验算", shear_context],
            ["上部计算纵筋As'", "", "", ""],
            ["下部计算纵筋As", "", "", ""],
            ["上部纵筋实配", "", "", ""],
            ["下部纵筋实配", "", "", ""],
            ["挠度限值", f"[f]={self.crack_width_limit:.2f}mm"],
            ["挠度验算结果", "满足"],
            ["裂缝宽度", "", "", ""],
            ["裂缝限值", f"[ω]={self.crack_width_limit:.2f}mm"],
            ["裂缝验算结果", ""],
        ]
        cal_table.set_table_context(table_context)
        self.add_table(cal_table)
        table_index += 1

        if (
            self.current_stair.stair_type == "CT"
            or self.current_stair.stair_type == "DT"
        ):
            right_table = DocTable(8, 4)
            right_table.merge_cells(3, 1, 3, 3)
            right_table.set_table_title(
                f"计算板段-{table_index}(右延伸段)：截面B×H=1000mm×{self.current_stair.right_thick}mm"
            )

            moments = self.current_stair.get_right_slab_table_moments()
            shears = self.current_stair.get_right_slab_table_shears()
            shear_context = self.current_stair.get_shear_validate(
                "right", ft, self.cover_thickness
            )

            table_context = [
                ["截面位置", "左", "中", "右"],
                [
                    "弯矩(M)",
                    f"{moments[0]:.2f}",
                    f"{moments[1]:.2f}",
                    f"{moments[2]:.2f}",
                ],
                ["剪力(V)", f"{shears[0]:.2f}", f"{shears[1]:.2f}", f"{shears[2]:.2f}"],
                ["抗剪截面验算", shear_context],
                ["上部计算纵筋As'", "", "", ""],
                ["下部计算纵筋As", "", "", ""],
                ["上部纵筋实配", "", "", ""],
                ["下部纵筋实配", "", "", ""],
            ]
            right_table.set_table_context(table_context)

            self.add_table(right_table)
            table_index += 1

    def create(self):
        self.add_first_part()
        for i in range(len(self.stair_list)):
            stair = self.stair_list[i]
            self.add_single_stair(stair, i + 1)
            if i < len(self.stair_list) - 1:
                self.add_page_break()

    def save_to_file(self, path):
        self.save(path)



from typing import List
import sqlite3
import os


class Connector:

    def __init__(self, file_path=None):
        self.set_db_file(file_path)
        self.connection = None
        self.cursor = None

    def set_db_file(self, file_path):
        if file_path != None and not os.path.exists(file_path):
            raise AttributeError(
                "The file_path is not existed, please check your file path. "
            )
        self.__file_path = file_path

    def connect(self):
        """
        建立与 SQLite 数据库的连接
        """
        if self.__file_path == None:
            raise AttributeError(
                "The file_path is None, you should set db file path before connect. Try the method [set_db_file]. "
            )
        try:
            self.connection = sqlite3.connect(self.__file_path)
            self.cursor = self.connection.cursor()
        except sqlite3.Error as e:
            print(f"连接数据库时出错: {e}")

    def extract_table(self, table_name):
        """
        从指定的表中提取数据
        :param table_name: 要提取数据的表名
        :return: 表中的数据列表
        """
        if self.cursor is None:
            self.connect()
        try:
            self.cursor.execute(f"SELECT * FROM {table_name}")
            rows = self.cursor.fetchall()
            return rows
        except sqlite3.Error as e:
            print(f"从表 {table_name} 提取数据时出错: {e}")
            return []

    def extract_table_by_columns(self, table_name, column_list: List[str]):
        """
        从指定的表中提取数据
        :param table_name: 要提取数据的表名
        :param column_list: 要提取的列名集合
        :return: 表中的数据列表
        """
        if self.cursor is None:
            self.connect()
        try:
            column_list_sql = ",".join(column_list)
            self.cursor.execute(f"SELECT {column_list_sql} FROM {table_name}")
            rows = self.cursor.fetchall()
            return rows
        except sqlite3.Error as e:
            print(f"从表 {table_name} 提取数据时出错: {e}")
            return []

    def extract_table_by_columns_and_filter(
        self, table_name, column_list: List[str], limit_column: str, target_value
    ):
        if self.cursor is None:
            self.connect()
        try:
            column_list_sql = ",".join(column_list)
            self.cursor.execute(
                f"SELECT {column_list_sql} FROM {table_name} WHERE {limit_column}=={target_value}"
            )
            rows = self.cursor.fetchall()
            return rows
        except sqlite3.Error as e:
            print(f"从表 {table_name} 提取数据时出错: {e}")
            return []

    def is_table_in_db(self, table_name: str):

        if self.cursor is None:
            self.connect()
        try:
            self.cursor.execute(
                "SELECT name FROM sqlite_master WHERE type='table' AND name=?",
                (table_name,),
            )
            result = self.cursor.fetchone()
            return result
        except sqlite3.Error as e:
            print(f"从表 {table_name} 提取数据时出错: {e}")
            return []

    def close(self):
        """
        关闭数据库连接
        """
        if self.connection:
            self.connection.close()



import math
import abc
from typing import List
import matplotlib.pyplot as plt


class SinglePeriod:
    """单个周期"""

    def __init__(
        self,
        index: int,
        time: float,
        angle: float = 0,
        coeff_x: float = 1,
        coeff_y: float = 0,
        coeff_z: float = 0,
        mass_particpate_x: float = 0,
        mass_particpate_y: float = 0,
        mass_particpate_z: float = 0,
    ):
        self.index = index
        self.time = time
        self.angle = angle
        assert (
            abs(coeff_x + coeff_y + coeff_z - 1) < 0.01
        ), "The sum of three participite coeff should == 1"
        self.coeff_x = coeff_x
        self.coeff_y = coeff_y
        self.coeff_z = coeff_z
        if coeff_x >= coeff_y and coeff_x >= coeff_z:
            self.direction = "X"
        elif coeff_y >= coeff_x and coeff_y >= coeff_z:
            self.direction = "Y"
        else:
            self.direction = "Z"
        self.mass_participate_x = mass_particpate_x
        self.mass_participate_y = mass_particpate_y
        self.mass_participate_z = mass_particpate_z

    @property
    def time_str(self):
        return f"{self.time:.4f}"

    @property
    def movement_coeff(self):
        return (
            f"{self.coeff_x + self.coeff_y:.2f} ({self.coeff_x:.2f}+{self.coeff_y:.2f})"
        )

    @property
    def rotation_coeff(self):
        return f"{self.coeff_z:.2f}"

    def get_mass_participate_x(self, last):
        return f"{self.mass_participate_x*100:.1f} ({(self.mass_participate_x + last)*100:.1f})"

    def get_mass_participate_y(self, last):
        return f"{self.mass_participate_y*100:.1f} ({(self.mass_participate_y + last)*100:.1f})"

    def __str__(self):
        return f"T{self.index}:\t{self.time:.4f}s\t[X:{self.coeff_x*100:.1f}%;\tY:{self.coeff_y*100:.1f}%;\tZ:{self.coeff_z*100:.1f}%]"

    def __repr__(self):
        return str(self)

    def to_json(self):
        return {"index": self.index, "time": self.time}


class Period:
    def __init__(self, periods: List[SinglePeriod], model_type=None):
        if len(periods) < 3:
            raise ValueError("At least three periods is needed.")
        self.periods = periods

    def __str__(self):
        if len(self.periods) <= 10:
            return "\n".join([str(period) for period in self.periods])
        else:
            result = "\n".join([str(period) for period in self.periods[:9]])
            result += "\n....\n"
            result += str(self.periods[-1])
            return result

    def __repr__(self):
        return self.__str__()

    @classmethod
    def mock_data(
        cls, order: str = "xyz", num: int = 40, mass_participate: float = 0.95
    ):
        single_period_list = []
        single_period_list.append(
            SinglePeriod(
                1,
                2.5,
                0,
                order[0] == "x",
                order[0] == "y",
                order[0] == "z",
                0.1,
                0.1,
                0.1,
            )
        )
        single_period_list.append(
            SinglePeriod(
                2,
                2.0,
                0,
                order[1] == "x",
                order[1] == "y",
                order[1] == "z",
                0.2,
                0.2,
                0.2,
            )
        )
        single_period_list.append(
            SinglePeriod(
                3,
                1.5,
                0,
                order[2] == "x",
                order[2] == "y",
                order[2] == "z",
                0.3,
                0.3,
                0.3,
            )
        )

        for i in range(num - 3):
            single_period_list.append(
                SinglePeriod(
                    i + 4,
                    0.5 - 0.01 * i,
                    0,
                    1,
                    0,
                    0,
                    mass_participate,
                    mass_participate,
                    mass_participate,
                )
            )
        return Period(single_period_list)

    def to_json(self):
        return {"periods": [i.to_json() for i in self.periods]}


class ValuePeer:
    def __init__(self, x: float, y: float):
        self.x = x
        self.y = y

    def __str__(self):
        if self.x > 500:
            return f"X:{self.x:.1f}\tY:{self.y:.1f}"
        elif self.x > 5:
            return f"X:{self.x:.2f}\tY:{self.y:.2f}"
        else:
            return f"X:{self.x:.3f}\tY:{self.y:.3f}"

    def to_json(self):
        return {"X": self.x, "Y": self.y}


class FloorDrift:
    def __init__(
        self,
        floor_height: float,
        drift_max_disp_x: float,
        drift_max_disp_y: float,
        drift_min_disp_x: float,
        drift_min_disp_y: float,
        max_disp_x: float,
        ave_disp_x: float,
        max_disp_y: float,
        ave_disp_y: float,
    ):
        self.floor_height = floor_height
        self.drift_max_disp_x = drift_max_disp_x
        self.drift_max_disp_y = drift_max_disp_y
        self.drift_min_disp_x = drift_min_disp_x
        self.drift_min_disp_y = drift_min_disp_y
        self.max_disp_x = max_disp_x
        self.ave_disp_x = ave_disp_x
        self.max_disp_y = max_disp_y
        self.ave_disp_y = ave_disp_y

    @property
    def drift_x(self):
        return round(self.floor_height / self.drift_max_disp_x)

    @property
    def drift_y(self):
        return round(self.floor_height / self.drift_max_disp_y)


class FloorSeismicResult:
    def __init__(
        self,
        floor_num: int,
        tower_num: int,
        force: ValuePeer = None,
        shear: ValuePeer = None,
        moment: ValuePeer = None,
        disp: ValuePeer = None,
        stiffness: List[ValuePeer] = [],
        shear_capacity: ValuePeer = None,
        drifts: List[FloorDrift] = [],
    ):
        self.floor_num = floor_num
        self.tower_num = tower_num
        self.force = force
        self.shear = shear
        self.moment = moment
        self.disp = disp
        self.stiffness = stiffness
        self.shear_capacity = shear_capacity
        self.drifts = drifts

    def __str__(self):
        return f"Flr.{self.floor_num}:Fx={self.force.x};Fy={self.force.y}"

    def __repr__(self):
        return self.__str__()

    def to_json(self):
        return {
            "floor_num": self.floor_num,
            "tower_num": self.tower_num,
            "shear": self.shear.to_json(),
        }


class SeismicResult:
    def __init__(self, floor_result: List[FloorSeismicResult]):
        self.floor_result = floor_result

    @property
    def seismic_shear_x(self):
        return [i.shear.x for i in self.floor_result]

    @property
    def floor_index(self):
        return [i + 1 for i in range(len(self.floor_result))]

    def plot_shear(self):
        fig, ax = plt.subplots(figsize=(2, 5))
        ax.plot(self.seismic_shear_x, self.floor_index)
        return fig, ax

    def __str__(self):
        result = f"Total floor: {len(self.floor_result)}\n"
        for temp_result in self.floor_result:
            result += str(temp_result)
            result += "\n"
        return result

    def __repr__(self):
        return self.__str__()

    def to_json(self):
        return {"floor_result": [i.to_json() for i in self.floor_result]}


if __name__ == "__main__":
    p_list = []
    for i in range(112):
        p_list.append(SinglePeriod(i + 1, i * 0.1 + 0.1, 0, 1 - i * 0.1, i * 0.1, 0))
    P = Period(p_list)
    print(str(P))



from typing import List
from ..YDBLoader.BuildingDefine import Period


X_DIRECTION = "X向"
Y_DIRECTION = "Y向"


class ChapterTemplate:
    def __init__(
        self,
        title,
        paragraph: List,
        table=None,
        picture=None,
        table_context=None,
        *args,
        **kwargs,
    ):
        self.title = title
        self.paragraph = paragraph
        self.table = table
        self.picture = picture
        self.table_context = table_context


def period_para_analysis(period_result: Period, chapter_index: int, sub_index: int):
    first_para = __period_first_para(period_result, chapter_index, sub_index)
    second_para = __period_second_para(period_result)
    return [first_para, second_para]


def __period_first_para(period_result: Period, chapter_index: int, sub_index: int):
    type_1 = period_result.periods[0].direction
    type_2 = period_result.periods[1].direction
    type_3 = period_result.periods[2].direction
    first_para = (
        f"表{chapter_index}.{sub_index}.1为强制刚性楼板假定下计算得到的结构模态信息，"
    )
    if type_1.upper() == "X" and type_2.upper() == "Y" and type_3.upper() == "Z":
        z_x = period_result.periods[2].time / period_result.periods[0].time
        z_y = period_result.periods[2].time / period_result.periods[1].time
        first_para += (
            "前两阶振型分别为X、Y向平动振型，第三阶振型为Z向扭转振型；"
            + f"结构第一扭转周期与第一、第二平动周期的比值分别为{z_x:.2f}和{z_y:.2f}。"
        )
    elif type_1.upper() == "Y" and type_2.upper() == "X" and type_3.upper() == "Z":
        z_y = period_result.periods[2].time / period_result.periods[0].time
        z_x = period_result.periods[2].time / period_result.periods[1].time
        first_para += (
            "前两阶振型分别为Y、X向平动振型，第三阶振型为Z向扭转振型；"
            + f"结构第一扭转周期与第一、第二平动周期的比值分别为{z_y:.2f}和{z_x:.2f}。"
        )
    elif type_1.upper() == "X" and type_2.upper() == "Z" and type_3.upper() == "Y":
        z_x = period_result.periods[1].time / period_result.periods[0].time
        first_para += (
            "第一阶振型为X向平动，第二阶振型为Z向扭转，第三阶振型为Y向平动；"
            + f"结构第一扭转周期与第一平动周期的比值分别为{z_x:.2f}。"
        )
    elif type_1.upper() == "Y" and type_2.upper() == "Z" and type_3.upper() == "X":
        z_y = period_result.periods[1].time / period_result.periods[0].time
        first_para += (
            "第一阶振型为Y向平动，第二阶振型为Z向扭转，第三振阶型为X向平动；"
            + f"结构第一扭转周期与第一平动周期的比值分别为{z_y:.2f}。"
        )
    elif type_1.upper() == "Z" and type_2.upper() == "X" and type_3.upper() == "Y":
        first_para += "*{第一阶振型为Z向扭转，第二、三阶振型分别为X、Y向平动，请注意复核计算结果；}"
    else:
        first_para += "*{第一阶振型为Z向扭转，第二、三阶振型分别为Y、X向平动，请注意复核计算结果；}"
    first_para += f"结构前三振型如图{chapter_index}.{sub_index}.1所示。"
    return first_para


def __period_second_para(period_result: Period):
    total_participate_x = sum([p.mass_participate_x for p in period_result.periods])
    total_participate_y = sum([p.mass_participate_y for p in period_result.periods])
    if total_participate_x >= 0.9 and total_participate_y >= 0.9:
        second_para = (
            "从上表可以看出，X、Y方向平动的质量参与系数均大于90%，满足规范要求。"
        )
    elif total_participate_x >= 0.9:
        second_para = "从上表可以看出，X方向平动的质量参与系数大于90%，*{但Y方向不满足，可酌情增加计算振型数量。}"
    elif total_participate_y >= 0.9:
        second_para = "从上表可以看出，Y方向平动的质量参与系数大于90%，*{但X方向不满足，可酌情增加计算振型数量。}"
    else:
        second_para = "从上表可以看出，*{X、Y方向平动的质量参与系数均小于90%，可酌情增加计算振型数量。}"
    return second_para


def shear_mass_ratio(chapter_index: int, sub_index: int):
    # todo: 这里需要根据设防烈度及基底剪力结果进行判断
    para_context = "剪重比为结构设计的重要指标，按规范7度，0.10g的反应谱计算，"
    para_context += f"结构基底和各楼层剪重比计算具体结果分别见表{chapter_index}.{sub_index}.1和图{chapter_index}.{sub_index}.1。"
    para_context += (
        "计算结果显示，结构两个主向的基底剪力均满足规范1.60%的最小剪重比要求。"
    )

    return para_context


def shear_and_moment(chapter_index: int, sub_index: int):
    para = f"塔楼基底剪力和基底倾覆力矩见下表{chapter_index}.{sub_index}.1。"
    para += f"图{chapter_index}.{sub_index}.1为50年重现期下的风荷载和小震作用下的楼层剪力分布，"
    para += f"图{chapter_index}.{sub_index}.2为50年重现期下的风荷载和小震作用下的倾覆弯矩分布。"
    para += "计算结果显示，"
    para += "在X、Y两个方向，小震作用下的楼层剪力和倾覆弯矩均远大于50年重现期下的风荷载作用，"
    para += "也表明在结构刚度和变形方面，小震起控制作用。"

    return para


def horizental_moment_ratio_for_column(chapter_index: int, sub_index: int):
    para1 = f"图{chapter_index}.{sub_index}.1为结构在双向水平地震作用下，各层框架柱分担的倾覆力矩与各层结构总倾覆力矩的比值（倾覆力矩分担比），"
    para1 += f"表{chapter_index}.{sub_index}.1与表{chapter_index}.{sub_index}.2为结构分别在双向水平地震作用下，各层框架柱分担的剪力、倾覆力矩及其所占比例的具体数值。"

    para2 = "结果表明：在规定水平力下，结构底层框架部分承受的地震倾覆力矩分担比"
    para2 += "在X向和Y向分别为54.8%和48.0%。底层倾覆力矩分担比大于50%，"
    para2 += "按照《高规》第8.1.3条第3款之规定，本工程应按照框架-剪力墙结构进行设计，"
    para2 += "其最大适用高度可比框架结构适当增加，框架部分的抗震等级和轴压比限值宜按框架结构的规定采用。"

    return [para1, para2]


def disp_and_drift(chapter_index: int, sub_index: int):
    para = f"结构在小震与风荷载作用下的最大层间位移角如表{chapter_index}.{sub_index}.1和图{chapter_index}.{sub_index}.1所示。"
    para += "各工况下各楼层位移角均小于规范限值1/800的要求。"
    para += "此外，风荷载下各层层间位移角小于小震作用，小震起控制作用。"

    return para


class SRTemplate:
    # 前情提要
    FIRST_INFO = (
        lambda model_name: f"*{{本报告内容针对模型“{model_name}”，请注意核对模型名称！}}"
    )
    # 小震章节提要
    SEISMIC_CHAPTER_TITLE = ChapterTemplate(
        title=lambda index: f"{index}.小震弹性分析的主要结果",
        paragraph=lambda index, yjk_version: [
            f"本模型弹性计算分析和构件设计软件采用YJK-{yjk_version}，结构计算模型如图{index}.0.1所示，结构计算假定如下：",
            "1. *{XXXX作为上部结构的嵌固端；}",
            "2. 计算结构整体指标按刚性板假定，构件验算时按弹性板设计。",
            "*{(这里需要一张图片！)}",
        ],
        picture=lambda index: f"图{index}.0.1 YJK计算模型示意图",
    )
    # 嵌固层
    SEISMIC_EMBEDDING = ChapterTemplate(
        title=lambda index, sub_index: f"{index}.{sub_index} 嵌固层",
        paragraph=lambda index, sub_index: [
            "采用《高层建筑混凝土结构技术规程》附录E剪切刚度的计算公式，"
            + "计算地下室顶板上下两层的侧向刚度比值。地下室范围的墙、柱构件取至塔楼外三个梁跨。"
            + f"计算结果见表{index}.{sub_index}.1，表明*{{地下室的刚度满足结构嵌固端的要求，"
            + "结构分析将嵌固端选取在地下室顶板是合理的。}",
        ],
        table=lambda index, sub_index: f"表{index}.{sub_index}.1 嵌固层验算",
        table_context=[
            [
                "-",
                "X向剪切刚度(kN/m)",
                "Y向剪切刚度(kN/m)",
                r"层高H_{i}(m)",
                "X向剪切刚度比",
                "Y向剪切刚度比",
                "结论",
            ],
            [
                "首层",
                "-",
                "-",
                "-",
                "-",
                "-",
                "-",
            ],
            [
                "地下一层",
                "-",
                "-",
                "-",
                "-",
                "-",
                "-",
            ],
        ],
    )

    # 结构质量
    PROJECT_MASS = ChapterTemplate(
        title=lambda index, sub_index: f"{index}.{sub_index} 结构质量",
        paragraph=lambda index, sub_index, **kwargs: [
            f"本塔楼结构重力荷载代表值为{kwargs["total_mass"]}吨，"
            + f"地上部分的结构楼板面积为{kwargs["total_area"]}平方米，"
            + f"按结构楼板折算的重量约为{kwargs["average_load"]}kN/m^{{2}}。"
            + f"其中恒载及活载详情见表{index}.{sub_index}.1。",
        ],
        table=lambda index, sub_index: f"表{index}.{sub_index}.1 结构质量组成",
        table_context=lambda **kwargs: [
            [
                "类别",
                "数值(t)",
                "占比",
                "单位楼板面积重量\r（kN/m^{2})",
            ],
            [
                "恒载",
                f"{kwargs['dead_mass']}",
                f"{kwargs['dead_percentage']}",
                f"{kwargs['dead_average']}",
            ],
            [
                "活载*0.5",
                f"{kwargs['live_mass']}",
                f"{kwargs['live_percentage']}",
                f"{kwargs['live_average']}",
            ],
            [
                "总质量(D+0.5L)",
                f"{kwargs['total_mass']}",
                f"{kwargs['total_percentage']}",
                f"{kwargs['total_average']}",
            ],
        ],
    )

    # 振型与周期
    PERIOD = ChapterTemplate(
        title=lambda index, sub_index: f"{index}.{sub_index} 振型与周期",
        paragraph=period_para_analysis,
        table=lambda index, sub_index: f"表{index}.{sub_index}.1 结构模态信息",
        table_context=[
            [
                "振型号",
                "周期",
                "平动系数(X+Y)",
                "扭转系数",
                "X向平动质量参与系数(累计%)",
                "Y向平动质量参与系数(累计%)",
            ],
        ],
        picture=lambda index, sub_index: f"图{index}.{sub_index}.1 结构前三振型示意图",
    )

    # 楼层剪重比
    SHEAR_MASS_RATIO = ChapterTemplate(
        title=lambda index, sub_index: f"{index}.{sub_index} 楼层剪重比",
        paragraph=shear_mass_ratio,
        table=lambda index, sub_index: f"表{index}.{sub_index}.1 结构基底剪重比",
        table_context=lambda **kwargs: [
            [
                "方向",
                "基底剪力(kN)",
                "重力荷载代表值(kN)",
                "剪重比",
            ],
            [
                X_DIRECTION,
                "-",
                "-",
                "-",
            ],
            [
                Y_DIRECTION,
                "-",
                "-",
                "-",
            ],
        ],
        picture=lambda index, sub_index: f"图{index}.{sub_index}.1 地震作用下各楼层剪重比",
    )

    # 楼层剪力及倾覆力矩
    SHEAR_AND_MOMENT = ChapterTemplate(
        title=lambda index, sub_index: f"{index}.{sub_index} 楼层剪力及倾覆力矩",
        paragraph=shear_and_moment,
        table=lambda index, sub_index: f"表{index}.{sub_index}.1 结构基底总剪力和倾覆力矩",
        table_context=lambda **kwargs: [
            [
                "--",
                "--",
                "YJK",
                "YJK",
            ],
            [
                "--",
                "项目",
                X_DIRECTION,
                Y_DIRECTION,
            ],
            [
                "风荷载",
                "基底剪力(kN)",
                "-",
                "-",
            ],
            [
                "风荷载",
                "基地倾覆力矩(MN·m)",
                "-",
                "-",
            ],
            [
                "多遇地震",
                "基底剪力(kN)",
                "-",
                "-",
            ],
            [
                "多遇地震",
                "基地倾覆力矩(MN·m)",
                "-",
                "-",
            ],
        ],
        picture=lambda index, sub_index: [
            f"图{index}.{sub_index}.1 风荷载和小震作用下的楼层剪力",
            f"图{index}.{sub_index}.2 风荷载和小震作用下的楼层倾覆力矩",
        ],
    )

    # 框架倾覆力矩占比
    HORIZENTAL_MOMENT_RATIO_FOR_COLUMN = ChapterTemplate(
        title=lambda index, sub_index: f"{index}.{sub_index} 框架倾覆力矩占比",
        paragraph=horizental_moment_ratio_for_column,
        table=lambda index, sub_index: [
            f"表{index}.{sub_index}.1 外框柱剪力及倾覆力矩分担比（X向）",
            f"表{index}.{sub_index}.2 外框柱剪力及倾覆力矩分担比（Y向）",
        ],
        table_context=lambda **kwargs: [
            [
                "层号",
                "剪力(kN)",
                "剪力分担比",
                "弯矩(kN*m)",
                "弯矩分担比",
            ],
        ],
        picture=lambda index, sub_index: f"图{index}.{sub_index}.1 各楼层的框架剪力和弯矩分比",
    )

    # 结构位移与层间位移
    DISP_AND_DRIFT = ChapterTemplate(
        title=lambda index, sub_index: f"{index}.{sub_index} 结构位移与层间位移",
        paragraph=disp_and_drift,
        table=lambda index, sub_index: f"表{index}.{sub_index}.1 楼层最大层间位移角",
        table_context=lambda **kwargs: [
            [
                "类别",
                "数值(t)",
                "占比",
                "单位楼板面积重量\r（kN/m^{2})",
            ],
            [
                "恒载",
                "-",
                "-",
                "-",
            ],
            [
                "活载*0.5",
                "-",
                "-",
                "-",
            ],
            [
                "总质量(D+0.5L)",
                "-",
                "-",
                "-",
            ],
        ],
    )

    # 侧向刚度比
    HORIZENTAL_STIFFNESS_RATIO = ChapterTemplate(
        title=lambda index, sub_index: f"{index}.{sub_index} 侧向刚度比",
        paragraph=lambda index, sub_index, **kwargs: [
            f"本塔楼结构重力荷载代表值为{kwargs["total_mass"]}吨，"
            + f"地上部分的结构楼板面积为{kwargs["total_area"]}平方米，"
            + f"按结构楼板折算的重量约为{kwargs["average_load"]:.2f}kN/m2。"
            + f"其中恒载及活载详情见表{index}.{sub_index}.1。",
        ],
        table=lambda index, sub_index: f"表{index}.{sub_index}.1 结构质量组成",
        picture=lambda index, sub_index: f"图{index}.{sub_index}.1 结构楼层层间位移角",
        table_context=lambda **kwargs: [
            [
                "类别",
                "数值(t)",
                "占比",
                "单位楼板面积重量\r（kN/m^{2})",
            ],
            [
                "恒载",
                "-",
                "-",
                "-",
            ],
            [
                "活载*0.5",
                "-",
                "-",
                "-",
            ],
            [
                "总质量(D+0.5L)",
                "-",
                "-",
                "-",
            ],
        ],
    )

    # 扭转位移比
    ROTATION_RATIO = ChapterTemplate(
        title=lambda index, sub_index: f"{index}.{sub_index} 扭转位移比",
        paragraph=lambda index, sub_index, **kwargs: [
            f"本塔楼结构重力荷载代表值为{kwargs["total_mass"]}吨，"
            + f"地上部分的结构楼板面积为{kwargs["total_area"]}平方米，"
            + f"按结构楼板折算的重量约为{kwargs["average_load"]:.2f}kN/m2。"
            + f"其中恒载及活载详情见表{index}.{sub_index}.1。",
        ],
        table=lambda index, sub_index: f"表{index}.{sub_index}.1 结构质量组成",
        table_context=lambda **kwargs: [
            [
                "类别",
                "数值(t)",
                "占比",
                "单位楼板面积重量\r（kN/m^{2})",
            ],
            [
                "恒载",
                "-",
                "-",
                "-",
            ],
            [
                "活载*0.5",
                "-",
                "-",
                "-",
            ],
            [
                "总质量(D+0.5L)",
                "-",
                "-",
                "-",
            ],
        ],
    )
    # 刚重比
    STIFFNESS_MASS_RATIO = ChapterTemplate(
        title=lambda index, sub_index: f"{index}.{sub_index} 刚重比",
        paragraph=lambda index, sub_index, **kwargs: [
            f"本塔楼结构重力荷载代表值为{kwargs["total_mass"]}吨，"
            + f"地上部分的结构楼板面积为{kwargs["total_area"]}平方米，"
            + f"按结构楼板折算的重量约为{kwargs["average_load"]:.2f}kN/m2。"
            + f"其中恒载及活载详情见表{index}.{sub_index}.1。",
        ],
        table=lambda index, sub_index: f"表{index}.{sub_index}.1 结构质量组成",
        table_context=lambda **kwargs: [
            [
                "类别",
                "数值(t)",
                "占比",
                "单位楼板面积重量\r（kN/m^{2})",
            ],
            [
                "恒载",
                "-",
                "-",
                "-",
            ],
            [
                "活载*0.5",
                "-",
                "-",
                "-",
            ],
            [
                "总质量(D+0.5L)",
                "-",
                "-",
                "-",
            ],
        ],
    )
    # 楼层侧向受剪承载力比
    SHEAR_CAPACITY_RATIO = ChapterTemplate(
        title=lambda index, sub_index: f"{index}.{sub_index} 楼层侧向受剪承载力比",
        paragraph=lambda index, sub_index, **kwargs: [
            f"本塔楼结构重力荷载代表值为{kwargs["total_mass"]}吨，"
            + f"地上部分的结构楼板面积为{kwargs["total_area"]}平方米，"
            + f"按结构楼板折算的重量约为{kwargs["average_load"]:.2f}kN/m2。"
            + f"其中恒载及活载详情见表{index}.{sub_index}.1。",
        ],
        table=lambda index, sub_index: f"表{index}.{sub_index}.1 结构质量组成",
        table_context=lambda **kwargs: [
            [
                "类别",
                "数值(t)",
                "占比",
                "单位楼板面积重量\r（kN/m^{2})",
            ],
            [
                "恒载",
                "-",
                "-",
                "-",
            ],
            [
                "活载*0.5",
                "-",
                "-",
                "-",
            ],
            [
                "总质量(D+0.5L)",
                "-",
                "-",
                "-",
            ],
        ],
    )
    # 风荷载下的结构舒适度验算
    WIND_ACC = ChapterTemplate(
        title=lambda index, sub_index: f"{index}.{sub_index} 风荷载下的结构舒适度验算",
        paragraph=lambda index, sub_index, **kwargs: [
            f"本塔楼结构重力荷载代表值为{kwargs["total_mass"]}吨，"
            + f"地上部分的结构楼板面积为{kwargs["total_area"]}平方米，"
            + f"按结构楼板折算的重量约为{kwargs["average_load"]:.2f}kN/m2。"
            + f"其中恒载及活载详情见表{index}.{sub_index}.1。",
        ],
        table=lambda index, sub_index: f"表{index}.{sub_index}.1 结构质量组成",
        table_context=lambda **kwargs: [
            [
                "类别",
                "数值(t)",
                "占比",
                "单位楼板面积重量\r（kN/m^{2})",
            ],
            [
                "恒载",
                "-",
                "-",
                "-",
            ],
            [
                "活载*0.5",
                "-",
                "-",
                "-",
            ],
            [
                "总质量(D+0.5L)",
                "-",
                "-",
                "-",
            ],
        ],
    )



from .BuildingDefine import Beam, Column, Joint, ComponentType, Grid
from .BuildingDefine import SinglePeriod, Period, MassResult, SingleMassResult
from .BuildingDefine import ValuePeer, FloorSeismicResult, SeismicResult, FloorDrift
from .BuildingDefine.Section import Section, ShapeEnum
from .SQLiteConnector import Connector, YDBTableName, RowDataFactory
from .YDBType import YDBType
import os
from typing import List, Dict


class YDBLoader:

    def __init__(self, file_name: str = None, ydb_type: YDBType = None):
        # default type is ModelType
        if not file_name.endswith(".ydb"):
            raise ValueError("Plase use file ends with .ybd!")
        self.connector = Connector(file_name)
        self.ydb_type = self.__check_ydb_type()

    def get_columns(self) -> List[Column]:
        columns = []
        sections = self.__get_sections(ComponentType.Column)
        row_data = self.connector.extract_table_by_columns(
            YDBTableName.COLUMN_TABLE_NAME, YDBTableName.COLUMN_TABLE_USEFUL_COLUMNS
        )
        for temp_column in row_data:
            temp_col_id = RowDataFactory.extract_int(temp_column, 0)
            joint_id = RowDataFactory.extract_int(temp_column, 1)
            sect_id = RowDataFactory.extract_int(temp_column, 2)
            joint = self.__find_joint_by_id(joint_id)
            sect = [s for s in sections if s.id == sect_id][0]
            new_column = Column(temp_col_id, joint, sect)
            columns.append(new_column)
        return columns

    def get_beams(self) -> List[Beam]:
        beams = []
        sections = self.__get_sections(ComponentType.Beam)
        row_data = self.connector.extract_table_by_columns(
            YDBTableName.BEAM_TABLE_NAME, YDBTableName.BEAM_TABLE_USEFUL_COLUMNS
        )
        for temp_beam in row_data:
            temp_beam_id = RowDataFactory.extract_int(temp_beam, 0)
            grid_id = RowDataFactory.extract_int(temp_beam, 1)
            grid = self.__find_grid_by_id(grid_id)
            sect_id = RowDataFactory.extract_int(temp_beam, 2)
            sect = [s for s in sections if s.id == sect_id][0]
            new_beam = Beam(temp_beam_id, grid.start_joint, grid.end_joint, sect)
            beams.append(new_beam)
        return beams

    def __get_sections(self, comp_type: ComponentType):
        table_name = ""
        table_columns = []
        table_names_for_different_comptype = {
            ComponentType.Column: [
                YDBTableName.COLUMN_SECTION_TABLE_NAME,
                YDBTableName.SECTION_TABLE_USEFUL_COLUMNS,
            ],
            ComponentType.Beam: [
                YDBTableName.BEAM_SECTION_TABLE_NAME,
                YDBTableName.SECTION_TABLE_USEFUL_COLUMNS,
            ],
        }
        if comp_type not in table_names_for_different_comptype.keys():
            raise ValueError(f"{comp_type.name} is not suppported yet.")
        # 这里根据不同的构件类型，进行不同的截面数据获取
        table_name = table_names_for_different_comptype[comp_type][0]
        table_columns = table_names_for_different_comptype[comp_type][1]
        row_data = self.connector.extract_table_by_columns(table_name, table_columns)
        sections = []
        for temp_section in row_data:
            temp_section_id = RowDataFactory.extract_int(temp_section, 0)
            # 这里的mat暂时没用到
            mat = RowDataFactory.extract_int(temp_section, 1)
            kind = RowDataFactory.extract_int(temp_section, 2)
            shape_val = RowDataFactory.extract_list(temp_section, 3)[1:]
            new_section = Section(
                temp_section_id, ShapeEnum.ConvertToShapeEnum(kind), shape_val, mat
            )
            sections.append(new_section)
        return sections

    def __get_joints(self) -> Dict[int, Joint]:
        if hasattr(self, "joint_list"):
            return self.joint_list
        table_name = YDBTableName.JOINT_TABLE_NAME
        useful_columns = YDBTableName.JOINT_TABLE_USEFUL_COLUMNS
        row_data = self.connector.extract_table_by_columns(table_name, useful_columns)
        joint_list = {}
        for temp_joint in row_data:
            temp_joint_id = RowDataFactory.extract_int(temp_joint, 0)
            x = RowDataFactory.extract_float(temp_joint, 1)
            y = RowDataFactory.extract_float(temp_joint, 2)
            std_flr_id = RowDataFactory.extract_int(temp_joint, 3)
            new_joint = Joint(temp_joint_id, x, y, std_flr_id)
            joint_list[temp_joint_id] = new_joint
        self.joint_list = joint_list
        return self.joint_list

    def __get_grids(self) -> Dict[int, Grid]:
        if hasattr(self, "grid_list"):
            return self.grid_list
        table_name = YDBTableName.GRID_TABLE_NAME
        useful_columns = YDBTableName.GRID_TABLE_USEFUL_COLUMNS
        row_data = self.connector.extract_table_by_columns(table_name, useful_columns)
        grid_list = {}
        for temp_grid in row_data:
            temp_grid_id = RowDataFactory.extract_int(temp_grid, 0)
            start_joint_id = RowDataFactory.extract_int(temp_grid, 1)
            end_joint_id = RowDataFactory.extract_int(temp_grid, 2)
            s_joint = self.__find_joint_by_id(start_joint_id)
            e_joint = self.__find_joint_by_id(end_joint_id)
            grid = Grid(temp_grid_id, s_joint, e_joint)
            grid_list[temp_grid_id] = grid
        self.grid_list = grid_list
        return self.grid_list

    def __check_ydb_type(self) -> YDBType:
        if self.connector.is_table_in_db(YDBTableName.JOINT_TABLE_NAME):
            return YDBType.ModelYDB
        if self.connector.is_table_in_db(YDBTableName.RESULT_PERIOD_TABLE):
            return YDBType.ResultYDB
        raise ValueError(
            "This ydb database is not Model YDB neither Result YDB. Please use correct ydb file."
        )

    def __find_joint_by_id(self, joint_id: int) -> Joint:
        joint_list = self.__get_joints()
        try:
            return joint_list[joint_id]
        except KeyError:
            raise ValueError(f"No Joint's ID is {joint_id}.")

    def __find_grid_by_id(self, grid_id: int) -> Grid:
        grid_list = self.__get_grids()
        try:
            return grid_list[grid_id]
        except KeyError:
            raise ValueError(f"No Joint's ID is {grid_id}.")

    def __check_result_model(self, extracting_data_type):
        if self.ydb_type != YDBType.ResultYDB:
            raise TypeError(
                "This model is not ResultYDB file, "
                + f"dont have {extracting_data_type} result, please retry."
            )

    def get_mass_result(self) -> MassResult:
        self.__check_result_model("mass")
        table_name = YDBTableName.RESULT_MASS_TABLE
        useful_columns = YDBTableName.RESULT_MASS_USEFUL_COLUMNS
        row_data = self.connector.extract_table_by_columns(table_name, useful_columns)
        mass_list = []
        for temp_mass in row_data:
            floor_num = RowDataFactory.convert_to_int(temp_mass[0])
            tower_num = RowDataFactory.convert_to_int(temp_mass[1])
            mass_info = RowDataFactory.extract_list(temp_mass, 2)
            dead_load = RowDataFactory.convert_to_float(mass_info[1])
            live_load = RowDataFactory.convert_to_float(mass_info[2])
            #  TODO: 需要计算slab的结果
            slab_area = 10
            single_mass = SingleMassResult(
                floor_num, tower_num, dead_load, live_load, slab_area
            )
            mass_list.append(single_mass)
        return MassResult(mass_list)

    def get_period_result(self) -> Period:
        self.__check_result_model("period")
        table_name = YDBTableName.RESULT_PERIOD_TABLE
        useful_columns = YDBTableName.RESULT_PERIOD_USEFUL_COLUMNS
        row_data = self.connector.extract_table_by_columns(table_name, useful_columns)
        periods = []
        for temp_period in row_data:
            module_id = RowDataFactory.extract_int(temp_period, 0)
            if module_id != 1:
                continue
            period_index = RowDataFactory.extract_int(temp_period, 1)
            time = RowDataFactory.extract_float(temp_period, 2)
            angle = RowDataFactory.extract_float(temp_period, 3)
            coeff = RowDataFactory.extract_list(temp_period, 4)
            mass_participate = RowDataFactory.extract_list(temp_period, 5)
            period = SinglePeriod(
                period_index,
                time,
                angle,
                RowDataFactory.convert_to_float(coeff[1]),
                RowDataFactory.convert_to_float(coeff[2]),
                RowDataFactory.convert_to_float(coeff[-1]),
                RowDataFactory.convert_to_float(mass_participate[1]),
                RowDataFactory.convert_to_float(mass_participate[2]),
                RowDataFactory.convert_to_float(mass_participate[-1]),
            )
            periods.append(period)
        return Period(periods)

    def get_seismic_result(self) -> SeismicResult:
        self.__check_result_model("seismic")
        table_name = YDBTableName.RESULT_FLOOR_DATA_TABLE
        useful_columns = YDBTableName.RESULT_FLOOR_DATA_USEFUL_COLUMNS_SEISMIC
        useful_columns_stiffness = (
            YDBTableName.RESULT_FLOOR_DATA_USEFUL_COLUMNS_STIFFNESS
        )
        row_data = self.connector.extract_table_by_columns(table_name, useful_columns)
        stiffness_row_data = self.connector.extract_table_by_columns(
            table_name, useful_columns_stiffness
        )

        table_disp_name = YDBTableName.DISP_FLOOR_DATA_TABLE
        useful_columns_disp = YDBTableName.DISP_FLOOR_DATA_USEFUL_COLUMNS_WIND

        seismic_load_cases = self.__get_seismic_loadcase_numbers()

        seismic_disp_x_row_data = self.connector.extract_table_by_columns_and_filter(
            table_disp_name, useful_columns_disp, "LDCase", seismic_load_cases[0]
        )
        seismic_disp_y_row_data = self.connector.extract_table_by_columns_and_filter(
            table_disp_name, useful_columns_disp, "LDCase", seismic_load_cases[1]
        )
        jzb_coeff = self.__get_JZBCoe()
        floor_height = self.__get_floor_height()
        floor_result_list = []
        for i in range(len(row_data)):
            temp_floor = row_data[i]
            temp_floor_stiffness = stiffness_row_data[i]
            temp_disp_x = seismic_disp_x_row_data[i]
            temp_disp_y = seismic_disp_y_row_data[i]
            floor_num = RowDataFactory.extract_int(temp_floor, 0)
            tower_num = RowDataFactory.extract_int(temp_floor, 1)
            force_x = RowDataFactory.convert_to_float(
                RowDataFactory.extract_list(temp_floor, 2)[1]
            )
            force_y = RowDataFactory.convert_to_float(
                RowDataFactory.extract_list(temp_floor, 3)[1]
            )
            shear_x = RowDataFactory.convert_to_float(
                RowDataFactory.extract_list(temp_floor, 4)[1]
            )
            shear_y = RowDataFactory.convert_to_float(
                RowDataFactory.extract_list(temp_floor, 5)[1]
            )
            moment_x = RowDataFactory.convert_to_float(
                RowDataFactory.extract_list(temp_floor, 6)[1]
            )
            moment_y = RowDataFactory.convert_to_float(
                RowDataFactory.extract_list(temp_floor, 7)[1]
            )
            shear_cap_x = RowDataFactory.convert_to_float(
                RowDataFactory.extract_list(temp_floor, 8)[1]
            )
            shear_cap_y = RowDataFactory.convert_to_float(
                RowDataFactory.extract_list(temp_floor, 9)[1]
            )
            stiff_x_shear_cut = RowDataFactory.convert_to_float(
                RowDataFactory.extract_list(temp_floor_stiffness, 0)[1]
            )
            stiff_y_shear_cut = RowDataFactory.convert_to_float(
                RowDataFactory.extract_list(temp_floor_stiffness, 1)[1]
            )
            stiff_x_shear_dis = RowDataFactory.convert_to_float(
                RowDataFactory.extract_list(temp_floor_stiffness, 2)[1]
            )
            stiff_y_shear_dis = RowDataFactory.convert_to_float(
                RowDataFactory.extract_list(temp_floor_stiffness, 3)[1]
            )
            stiff_x_shear_bend = RowDataFactory.convert_to_float(
                RowDataFactory.extract_list(temp_floor_stiffness, 4)[1]
            )
            stiff_y_shear_bend = RowDataFactory.convert_to_float(
                RowDataFactory.extract_list(temp_floor_stiffness, 5)[1]
            )

            max_drift_disp_x = RowDataFactory.convert_to_float(temp_disp_x[3])
            max_drift_disp_y = RowDataFactory.convert_to_float(temp_disp_y[3])

            min_drift_disp_x = RowDataFactory.convert_to_float(temp_disp_x[4])
            min_drift_disp_y = RowDataFactory.convert_to_float(temp_disp_y[4])

            max_disp_x = RowDataFactory.convert_to_float(temp_disp_x[5])
            max_disp_y = RowDataFactory.convert_to_float(temp_disp_y[5])

            ave_disp_x = RowDataFactory.convert_to_float(temp_disp_x[6])
            ave_disp_y = RowDataFactory.convert_to_float(temp_disp_y[6])

            force = ValuePeer(force_x, force_y)
            shear = ValuePeer(shear_x, shear_y)
            moment = ValuePeer(moment_x, moment_y)
            disp = ValuePeer(
                max_disp_x * jzb_coeff[floor_num][0],
                max_disp_y * jzb_coeff[floor_num][1],
            )
            stiffness_list = [
                ValuePeer(stiff_x_shear_cut, stiff_y_shear_cut),
                ValuePeer(stiff_x_shear_dis, stiff_y_shear_dis),
                ValuePeer(stiff_x_shear_bend, stiff_y_shear_bend),
            ]
            shear_capacity = ValuePeer(shear_cap_x, shear_cap_y)
            drifts = [
                FloorDrift(
                    floor_height[floor_num],
                    max_drift_disp_x * jzb_coeff[floor_num][0],
                    max_drift_disp_y * jzb_coeff[floor_num][1],
                    min_drift_disp_x * jzb_coeff[floor_num][0],
                    min_drift_disp_y * jzb_coeff[floor_num][1],
                    max_disp_x * jzb_coeff[floor_num][0],
                    max_disp_y * jzb_coeff[floor_num][1],
                    ave_disp_x * jzb_coeff[floor_num][0],
                    ave_disp_y * jzb_coeff[floor_num][1],
                )
            ]
            temp_floor_result = FloorSeismicResult(
                floor_num,
                tower_num,
                force,
                shear,
                moment,
                disp,
                stiffness_list,
                shear_capacity,
                drifts,
            )
            floor_result_list.append(temp_floor_result)

        return SeismicResult(floor_result_list)

    def get_wind_result(self) -> SeismicResult:
        self.__check_result_model("wind")
        table_name = YDBTableName.RESULT_FLOOR_DATA_TABLE
        useful_columns = YDBTableName.RESULT_FLOOR_DATA_USEFUL_COLUMNS_WIND
        row_data = self.connector.extract_table_by_columns(table_name, useful_columns)

        table_disp_name = YDBTableName.DISP_FLOOR_DATA_TABLE
        useful_columns_disp = YDBTableName.DISP_FLOOR_DATA_USEFUL_COLUMNS_WIND

        wind_load_cases = self.__get_wind_loadcase_numbers()

        wind_disp_x_row_data = self.connector.extract_table_by_columns_and_filter(
            table_disp_name, useful_columns_disp, "LDCase", wind_load_cases[0]
        )
        wind_disp_y_row_data = self.connector.extract_table_by_columns_and_filter(
            table_disp_name, useful_columns_disp, "LDCase", wind_load_cases[1]
        )

        floor_result_list = []
        floor_height = self.__get_floor_height()

        for i in range(len(row_data)):
            temp_floor = row_data[i]
            temp_disp_x = wind_disp_x_row_data[i]
            temp_disp_y = wind_disp_y_row_data[i]
            floor_num = RowDataFactory.extract_int(temp_floor, 0)
            tower_num = RowDataFactory.extract_int(temp_floor, 1)
            force_x = RowDataFactory.convert_to_float(
                RowDataFactory.extract_list(temp_floor, 2)[1]
            )
            force_y = RowDataFactory.convert_to_float(
                RowDataFactory.extract_list(temp_floor, 2)[3]
            )
            shear_x = RowDataFactory.convert_to_float(
                RowDataFactory.extract_list(temp_floor, 3)[1]
            )
            shear_y = RowDataFactory.convert_to_float(
                RowDataFactory.extract_list(temp_floor, 3)[3]
            )
            moment_x = RowDataFactory.convert_to_float(
                RowDataFactory.extract_list(temp_floor, 4)[1]
            )
            moment_y = RowDataFactory.convert_to_float(
                RowDataFactory.extract_list(temp_floor, 4)[3]
            )

            max_drift_disp_x = RowDataFactory.convert_to_float(temp_disp_x[3])
            max_drift_disp_y = RowDataFactory.convert_to_float(temp_disp_y[3])

            min_drift_disp_x = RowDataFactory.convert_to_float(temp_disp_x[4])
            min_drift_disp_y = RowDataFactory.convert_to_float(temp_disp_y[4])

            max_disp_x = RowDataFactory.convert_to_float(temp_disp_x[5])
            max_disp_y = RowDataFactory.convert_to_float(temp_disp_y[5])

            ave_disp_x = RowDataFactory.convert_to_float(temp_disp_x[6])
            ave_disp_y = RowDataFactory.convert_to_float(temp_disp_y[6])

            force = ValuePeer(abs(force_x), abs(force_y))
            shear = ValuePeer(abs(shear_x), abs(shear_y))
            moment = ValuePeer(abs(moment_x), abs(moment_y))
            disp = ValuePeer(123, 245)
            drifts = [
                FloorDrift(
                    floor_height[floor_num],
                    max_drift_disp_x,
                    max_drift_disp_y,
                    min_drift_disp_x,
                    min_drift_disp_y,
                    max_disp_x,
                    max_disp_y,
                    ave_disp_x,
                    ave_disp_y,
                )
            ]
            temp_floor_result = FloorSeismicResult(
                floor_num, tower_num, force, shear, moment, disp, drifts=drifts
            )

            floor_result_list.append(temp_floor_result)
        return SeismicResult(floor_result_list)

    def __get_wind_loadcase_numbers(self):
        return [2, 4]

    def __get_seismic_loadcase_numbers(self):
        return [9, 10]

    def __get_JZBCoe(self):
        if hasattr(self, "jzb_coeff"):
            return self.jzb_coeff
        table_name = YDBTableName.ADJUST_COEF_TABLE
        useful_column = YDBTableName.ADJUST_COEF_USEFUL_COLUMN
        row_data = self.connector.extract_table_by_columns(table_name, useful_column)
        jzb_coeff = {}
        for temp_data in row_data:
            floor_num = RowDataFactory.extract_int(temp_data, 0)
            # tower_num = RowDataFactory.extract_int(temp_data, 1)
            coeff_x = RowDataFactory.convert_to_float(
                RowDataFactory.extract_list(temp_data, 2)[1]
            )
            coeff_y = RowDataFactory.convert_to_float(
                RowDataFactory.extract_list(temp_data, 3)[1]
            )
            jzb_coeff[floor_num] = [coeff_x, coeff_y]
        self.jzb_coeff = jzb_coeff
        return self.jzb_coeff

    def __get_floor_height(self):
        if hasattr(self, "floor_height"):
            return self.floor_height
        table_name = YDBTableName.REAL_FLOOR_TABLE
        useful_column = YDBTableName.REAL_FLOOR_USEFUL_COLUMN
        row_data = self.connector.extract_table_by_columns(table_name, useful_column)
        floor_height = {}
        for temp_data in row_data:
            floor_num = RowDataFactory.extract_int(temp_data, 0)
            height = RowDataFactory.extract_float(temp_data, 3)
            floor_height[floor_num] = height
        self.floor_height = floor_height
        return self.floor_height


if __name__ == "__main__":
    file_path = "testfiles/dtlmodel1.ydb"
    loader = YDBLoader(file_path)
    columns = loader.get_columns()

    for col in columns:
        print(col.section, col.joint)



from typing import List
import matplotlib.pyplot as plt
from matplotlib.axes import Axes
import io
import numpy as np

plt.rcParams["font.sans-serif"] = ["SimHei"]  # 用来正常显示中文标签
plt.rcParams["axes.unicode_minus"] = False
plt.rcParams["font.size"] = 14


def GetTicks(n_max=1):
    if n_max == 0:
        n_max = 100
    m = int(np.log10(n_max))
    if m <= 0 and n_max <= 1:
        m -= 1
    p = n_max / (10**m)
    if p <= 2.2:
        result = [i * 1 * 10**m for i in range(3)]
    elif p <= 3.5:
        result = [i * 2 * 10**m for i in range(3)]
    elif p <= 5:
        result = [i * 2.5 * 10**m for i in range(3)]
    elif p <= 6.9:
        result = [i * 3 * 10**m for i in range(4)]
    elif p <= 8:
        result = [i * 2 * 10**m for i in range(5)]
    else:
        result = [i * 2 * 10**m for i in range(6)]
    return result


class BasicPltPlotter:
    def __init__(self, fig_num: int = 1, fig_size=(10, 10)):
        self.fig, _axes = plt.subplots(1, fig_num, figsize=fig_size)
        if fig_num == 1:
            self.axes = [_axes]
        else:
            self.axes = _axes
        self.fig.patch.set_facecolor("none")
        for ax in self.axes:
            ax.patch.set_facecolor("none")
        self.axes: List[Axes]
        self.__remove_border()

    def __remove_border(self):
        for ax in self.axes:
            ax.spines["right"].set_color("none")
            ax.spines["top"].set_color("none")

    def save_to_stream(self):
        # 将图片保存到内存中的 BytesIO 对象
        img_buffer = io.BytesIO()
        self.fig.savefig(img_buffer, format="png", dpi=150)  # 保存为 PNG 格式
        plt.close()  # 关闭图形，释放内存
        # 将指针重置到流的开头，以便后续读取
        img_buffer.seek(0)
        return img_buffer


class SeismicPlotter(BasicPltPlotter):
    def __init__(self, fig_num=2, floor_num: int = 8):
        if fig_num != 1 and fig_num != 2:
            raise ValueError("Only 1 or 2 is accepted for fig_num.")
        if fig_num == 1:
            fig_size = (3, 5)
        else:
            fig_size = (6, 5)
        super().__init__(fig_num, fig_size)
        self.kwargs_x = {
            "label": "X",
            "ls": "-",
            "color": "k",
            "marker": "o",
            "ms": 3,
        }
        self.kwargs_y = {
            "label": "X",
            "ls": "-",
            "color": "r",
            "marker": "o",
            "ms": 3,
        }
        self.floor_num = floor_num
        self.y_label = "层号"
        self._y_values = [i + 1 for i in range(self.floor_num)]
        self._y_major_ticks = self.__create_y_ticks()
        self._y_minor_ticks = [i + 1 for i in range(self.floor_num)]
        self._ax1_x = [i for i in range(self.floor_num)]
        self._ax1_y = [i * 0.5 for i in range(self.floor_num)]
        self._ax2_x = [i for i in range(self.floor_num)]
        self._ax2_y = [i * 0.5 for i in range(self.floor_num)]

    def test_plot(self):
        self.__plot()

    def __plot(self):
        self.axes[0].plot(self._ax1_x, self._y_values, **self.kwargs_x)
        self.axes[0].plot(self._ax1_y, self._y_values, **self.kwargs_y)
        self.axes[1].plot(self._ax2_x, self._y_values, **self.kwargs_x)
        self.axes[1].plot(self._ax2_y, self._y_values, **self.kwargs_y)

    def __create_y_ticks(self):
        floor_num = self.floor_num
        return range(0, int(floor_num) + 1, int(floor_num // 5) + 1)

    def _validate_list_length(self, data_list, name):
        """
        验证列表长度是否与楼层数相等
        :param data_list: 待验证的列表
        :param name: 列表的名称，用于异常信息
        """
        if len(data_list) != self.floor_num:
            raise ValueError(
                f"Length of {name} is not equal to floor number: {self.floor_num}!"
            )



from .Joint import Joint


class Grid:
    def __init__(self, id: int, s_joint: Joint, e_joint: Joint):
        if s_joint.std_flr_id != e_joint.std_flr_id:
            raise ValueError(
                "The standard-floor-ids of two Joints is not same! "
                + "Please check the data!"
            )
        self.id = id
        self.start_joint = s_joint
        self.end_joint = e_joint
        self.std_flr_id = s_joint.std_flr_id

    def get_length(self):
        return self.start_joint.distance_to_joint(self.end_joint)



from ezdxf.enums import TextEntityAlignment
from CivilTools.Const import CADConst


class DrawingAttribs:
    """这是一个基础的属性，包含图层、颜色等所有人都有的信息"""

    def __init__(
        self,
        layer: str,
        color_index: int = CADConst.BY_LAYER,
    ):
        self.layer = layer
        self.color_index = color_index


class PolylineAttribs(DrawingAttribs):
    """Polyline对象的属性，包括是否封闭等"""

    def __init__(
        self,
        layer,
        color_index: int = CADConst.BY_LAYER,
        close: bool = False,
        constant_width: float = 0,
    ):
        super().__init__(layer, color_index)
        self.close = close
        self.constant_width = constant_width
        """全局宽度"""


class TextAttribs(DrawingAttribs):
    def __init__(
        self,
        layer,
        color_index: int = CADConst.BY_LAYER,
        text_height: float = 300,
        text_width_factor: float = 0.7,
        text_align: TextEntityAlignment = TextEntityAlignment.MIDDLE_CENTER,
    ):
        super().__init__(layer, color_index)
        self.text_height = text_height
        self.text_width_factor = text_width_factor
        self.text_align = text_align



from typing import List


class SingleMassResult:
    def __init__(
        self,
        floor_num: int,
        tower_num: int,
        dead_load: float,
        live_load: float,
        slab_area: float,
    ):
        self.floor_num = floor_num
        self.tower_num = tower_num
        self.dead_load = round(dead_load, 4)
        """单层恒载，单位kN，没有折减"""
        self.live_load = round(live_load, 4) * 2
        """单层活载，单位kN，没有折减"""
        self.slab_area = round(slab_area)
        """单层楼板面积，单位m2"""

    @property
    def total_load(self):
        """单层质量，恒+0.5活"""
        return round(self.dead_load + 0.5 * self.live_load, 4)

    def to_json(self):
        return {
            "floor_num": self.floor_num,
            "tower_num": self.tower_num,
            "dead_load": self.dead_load,
            "live_load": self.live_load,
        }


class MassResult:
    def __init__(self, mass_list: List[SingleMassResult]):
        self.mass_list = mass_list

    @property
    def total_slab_area(self):
        return sum([i.slab_area for i in self.mass_list])

    @property
    def total_dead_load(self):
        return sum([i.dead_load for i in self.mass_list])

    @property
    def total_live_load(self):
        return sum([i.live_load for i in self.mass_list])

    @property
    def total_load(self):
        return sum([i.total_load for i in self.mass_list])

    @classmethod
    def mock_data(cls):
        mass_list = []
        for i in range(5):
            mass_list.append(SingleMassResult(i + 1, 1, 5000, 2000, 350))

        return MassResult(mass_list)

    def to_json(self):
        return {"mass_list": [i.to_json() for i in self.mass_list]}



import ezdxf
from ezdxf.enums import TextEntityAlignment
from ezdxf.addons.drawing import RenderContext, Frontend
from ezdxf.addons.drawing.matplotlib import MatplotlibBackend
import matplotlib.pyplot as plt
import warnings
from math import inf
from typing import List, Iterable, Tuple
from .LayerManager import CADLayer, CADColor, CADLineType
from .DrawingAttribs import DrawingAttribs, PolylineAttribs, TextAttribs
from CivilTools.Const import CADConst


class BasicDXF:
    file_extension = ".dxf"
    DXF2007 = "AC1021"
    line_type_patterns = {
        CADLineType.Continuous: [1, 1],
        CADLineType.CENTER: [1, 0.4, -0.2, 0.1, -0.3],
        CADLineType.DASHDOT: [1, 0.6, -0.2, 0, -0.2],
        CADLineType.DASHED: [1, 0.7, -0.3],
    }
    """线型及其Pattern定义，Pattern定义可参考ezdxf内的定义
    The simple line type pattern is a list of
    floats :code:`[total_pattern_length, elem1, elem2, ...]`
    where an element > 0 is a line, an element < 0 is a gap and  an
    element == 0.0 is a dot.
    """
    DIM_STYLE_PREFIX = "CT"
    TEXT_STYLE_NAME = "CT2025"
    TEXT_SHX = "xd-hzs.shx"
    TEXT_BIG_SHX = "xd-hztxt.shx"

    def __init__(self):
        self.doc = ezdxf.new(BasicDXF.DXF2007)
        self.model_space = self.doc.modelspace()
        self.__loaded_line_types = []
        self.__min_point = [inf, inf]
        self.__max_point = [-inf, -inf]
        self.__load_text_type()
        self.__load_dim_type(100)

    def init_layers(self, layer_list: List[CADLayer]):
        layers = self.doc.layers
        for my_layer in layer_list:
            # 如果图层名称已存在，则不进行新增
            if my_layer.name in [l.dxf.name for l in layers]:
                warnings.warn(f"Layer {my_layer.name} already existed.")
                continue
            temp_layer = layers.new(name=my_layer.name)
            temp_layer.color = my_layer.color.value
            self.__load_line_type(
                my_layer.line_type.name, BasicDXF.line_type_patterns[my_layer.line_type]
            )
            temp_layer.dxf.linetype = my_layer.line_type.name
            self.doc.header["$CLAYER"] = my_layer.name

    def _add_horizental_line(
        self, start_x, start_y, length: float, attribs: PolylineAttribs | None = None
    ):
        self._add_polyline([[start_x, start_y], [start_x + length, start_y]], attribs)

    def _add_vertical_line(
        self, start_x, start_y, length: float, attribs: PolylineAttribs | None = None
    ):
        self._add_polyline([[start_x, start_y], [start_x, start_y + length]], attribs)

    def _add_rectangle(
        self,
        start_x,
        start_y,
        width: float,
        height: float,
        attribs: PolylineAttribs | None = None,
    ):
        pts = [
            [start_x, start_y],
            [start_x + width, start_y],
            [start_x + width, start_y + height],
            [start_x, start_y + height],
        ]
        attribs.close = True
        self._add_polyline(pts, attribs)

    def _add_polyline(
        self,
        points: Iterable[Tuple[float, float]],
        attribs: PolylineAttribs | None = None,
    ):
        polyline = self.model_space.add_lwpolyline(points, close=attribs.close)
        if attribs != None:
            polyline.dxf.layer = attribs.layer

        max_x = max([pt[0] for pt in points])
        max_y = max([pt[1] for pt in points])
        min_x = min([pt[0] for pt in points])
        min_y = min([pt[1] for pt in points])
        self.__update_boundary(max_x, max_y)
        self.__update_boundary(min_x, min_y)

    def _add_circle(
        self,
        center_point: Iterable[float],
        radius: float,
        attribs: DrawingAttribs | None = None,
    ):
        circle = self.model_space.add_circle(center_point, radius)
        if attribs != None:
            circle.dxf.layer = attribs.layer

    def _add_dimension(
        self,
        start_point: Iterable[float],
        end_point: Iterable[float],
        attribs: DrawingAttribs | None = None,
    ):
        dimension = self.model_space.add_linear_dim(
            (0, 0), start_point, end_point, dimstyle="CT-100"
        ).render()

        if attribs != None:
            # 这里.dimension才是Dimension对象，才有dxf属性
            dimension.dimension.dxf.layer = attribs.layer

    def _add_text(self, context: str, insert_point, attribs: TextAttribs):
        text = self.model_space.add_text(context, height=attribs.text_height)
        text.dxf.style = BasicDXF.TEXT_STYLE_NAME
        # width就是宽度因子系数
        text.dxf.width = attribs.text_width_factor
        text.dxf.layer = attribs.layer
        text.dxf.color = attribs.color_index
        text.set_placement(insert_point, align=attribs.text_align)
        max_x = insert_point[0]
        min_x = insert_point[0]
        max_y = insert_point[1] + attribs.text_height
        min_y = insert_point[1] - attribs.text_height
        self.__update_boundary(max_x, max_y)
        self.__update_boundary(min_x, min_y)

    def _save(self, path: str):
        self.__change_view()
        if not path.endswith(BasicDXF.file_extension):
            path += BasicDXF.file_extension
        for _ in range(10):
            try:
                self.doc.saveas(path)
            except Exception:
                path = path.replace(
                    BasicDXF.file_extension, "1" + BasicDXF.file_extension
                )

    def __change_view(self):
        if inf in self.__max_point or -inf in self.__min_point:
            return

        y_range = self.__max_point[1] - self.__min_point[1]
        x_range = self.__max_point[0] - self.__min_point[0]

        y_middle = (self.__max_point[1] + self.__min_point[1]) / 2
        # 为了使得内容靠右些，避免左侧panel占位的视觉影响
        x_middle = (self.__max_point[0] + self.__min_point[0] * 3) / 4
        # 乘以1.1的系数，增加了一些margin
        self.doc.set_modelspace_vport(
            max(x_range * 1.1, y_range * 1.1), (x_middle, y_middle)
        )

    def __update_boundary(self, x, y):
        temp_x1 = self.__min_point[0]
        temp_y1 = self.__min_point[1]
        self.__min_point = [min(temp_x1, x), min(temp_y1, y)]

        temp_x2 = self.__max_point[0]
        temp_y2 = self.__max_point[1]
        self.__max_point = [max(temp_x2, x), max(temp_y2, y)]

    def __load_line_type(self, name: str, pattern: List[float]):
        if name in self.__loaded_line_types:
            return
        self.doc.linetypes.add(name, pattern)
        self.__loaded_line_types.append(name)

    def __load_dim_type(self, scale: int):
        # 获取标注样式表
        dimstyles = self.doc.dimstyles
        # 定义新标注样式的名称
        new_dimstyle_name = f"{BasicDXF.DIM_STYLE_PREFIX}-{scale}"
        # 创建新的标注样式
        new_dimstyle = dimstyles.new(new_dimstyle_name)
        # 设置标注文字样式
        new_dimstyle.dxf.dimtxsty = BasicDXF.TEXT_STYLE_NAME
        # 设置标注文字高度
        new_dimstyle.dxf.dimtxt = 2.5
        # 设置箭头大小
        new_dimstyle.dxf.dimasz = 1
        # 设置标注线超出尺寸界线的长度
        new_dimstyle.dxf.dimexo = 0.625
        # 设置尺寸界线起点偏移量
        new_dimstyle.dxf.dimse1 = 0.625
        #  设置全局比例
        new_dimstyle.dxf.dimscale = scale
        # 保留至整数
        new_dimstyle.dxf.dimrnd = 1
        # 改为建筑箭头
        new_dimstyle.dxf.dimblk = "ARCHTICK"
        new_dimstyle.dxf.dimblk1 = "ARCHTICK"
        new_dimstyle.dxf.dimblk2 = "ARCHTICK"
        self.doc.header["$DIMSTYLE"] = new_dimstyle_name

    def __load_text_type(self):

        text_styles = self.doc.styles
        # 定义新文字样式的名称
        new_style_name = BasicDXF.TEXT_STYLE_NAME
        # 创建新的文字样式
        new_style = text_styles.new(new_style_name)
        # 设置文字样式的属性
        # 指定字体文件，例如 Arial 字体
        new_style.dxf.font = BasicDXF.TEXT_SHX
        new_style.dxf.bigfont = BasicDXF.TEXT_BIG_SHX
        # 设置文字高度
        new_style.dxf.height = 0
        new_style.dxf.width = 0.7
        # 修改当前选择的文字样式
        self.doc.header["$TEXTSTYLE"] = new_style_name

    def _remove_all_entities(self):
        self.model_space.delete_all_entities()
        self.__min_point = [inf, inf]
        self.__max_point = [-inf, -inf]

    def _repr_png_(self):
        """可以在jupyter中实时显示绘制的图像情况"""
        ctx = RenderContext(self.doc)
        fig, ax = plt.subplots()
        fig.patch.set_facecolor("black")
        ax.patch.set_facecolor("black")
        plt.axis("off")
        backend = MatplotlibBackend(ax)
        Frontend(ctx, backend).draw_layout(self.model_space)
        plt.show()



from .Beam import Beam
from .Section import Section
from .Geometry import Joint, Grid, StandFloor
from .Column import Column
from .Slab import Slab

from .ComponentType import ComponentType
from .GlobalResult import SingleMassResult, MassResult, Period, SinglePeriod
from .GlobalResult import ValuePeer, FloorSeismicResult, SeismicResult, FloorDrift



from ..Section import Section, ShapeEnum
from ..Geometry import Joint


class Column:
    def __init__(self, id: int, joint: Joint, section: Section):
        self.id = id
        self.joint = joint
        self.std_flr_id = joint.std_flr_id
        self.section = section

    def __str__(self):
        return f"Column:{self.id}-{str(self.section)}"

    def __repr__(self):
        return self.__str__()



from .BasicGenerator import BasicGenerator, PageSize
from .SeismicReport import SeismicReport, SeismicReportData
from .DocParagraph import DocParagraph
from .DocTable import DocTable
from .DocPicture import DocPicture
from .UtilFunctions import add_comma_in_num_str
from .StairCalculationReport import StairCalculationReport



from enum import Enum
import warnings


class ShapeEnum(Enum):
    # 矩形
    Rect = 1
    # 工字型
    HShape = 2
    # 圆形
    Circle = 3
    # 正多边形
    RegularPolygon = 4
    # 槽型
    Groove = 5
    # 十字形
    Cross = 6
    # 箱形
    Box = 7
    # 圆管
    CircleTube = 8
    # 钢管混凝土
    CircleCFT = 12
    # 工形劲
    HSRC = 13
    # 箱型劲
    BoxSRC = 14
    # 十字劲
    CrossSRC = 15

    # 未知截面均归于这个类别
    UnKnown = 999

    @classmethod
    def ConvertToShapeEnum(cls, index: int):
        try:
            return (ShapeEnum)(index)
        except ValueError:
            warnings.warn(
                f"Shape kind ${index} is not supported yet. It will be converted to UnKnown."
            )
            return ShapeEnum.UnKnown



from enum import Enum


class CADColor(Enum):
    Red = 1
    Yellow = 2
    Green = 3
    Cyan = 4
    Blue = 5
    Magenta = 6
    White = 7
    Gray = 8
    LightGray = 9


class CADLineType(Enum):
    Continuous = 1
    DASHED = 2
    CENTER = 3
    DASHDOT = 4


class CADLayer:
    def __init__(
        self,
        name: str,
        color: CADColor,
        line_type: CADLineType = CADLineType.Continuous,
    ):
        self.name = name
        self.color = color
        self.line_type = line_type


class DefaultLayers:
    TEXT = CADLayer("0S-TEXT", CADColor.Cyan)
    SYMBOL = CADLayer("0S-SYMB", CADColor.Red)



from enum import Enum

ID = "ID"
STD_FLR_ID = "StdFlrID"
JOINT_ID = "JtID"
JOINT_ID_1 = "Jt1ID"
JOINT_ID_2 = "Jt2ID"
GRID_ID = "GridID"
SECTION_ID = "SectID"
ECC = "Ecc"
ECC_X = "EccX"
ECC_Y = "EccY"
ROTATION = "Rotation"


FLOOR_NUM = "FlrNo"
TOWER_NUM = "TowNo"
LOAD_CASE_ID = "LDCase"


class YDBTableName:
    JOINT_TABLE_NAME = "tblJoint"
    JOINT_TABLE_USEFUL_COLUMNS = [ID, "X", "Y", STD_FLR_ID]

    GRID_TABLE_NAME = "tblGrid"
    GRID_TABLE_USEFUL_COLUMNS = [ID, JOINT_ID_1, JOINT_ID_2]
    """ 
    0-ID , 
    1-Joint1_ID , 
    2-Joint2_ID ,
    """

    COLUMN_SECTION_TABLE_NAME = "tblColSect"
    BEAM_SECTION_TABLE_NAME = "tblBeamSect"
    SECTION_TABLE_USEFUL_COLUMNS = [ID, "Mat", "Kind", "ShapeVal"]

    COLUMN_TABLE_NAME = "tblColSeg"
    COLUMN_TABLE_USEFUL_COLUMNS = [ID, JOINT_ID, SECTION_ID, ECC_X, ECC_Y, ROTATION]
    """ 
    0-ID , 
    1-Joint_ID , 
    2-Section_ID ,
    3-EccX ,
    4-EccY ,
    5-Rotation
    """

    BEAM_TABLE_NAME = "tblBeamSeg"
    BEAM_TABLE_USEFUL_COLUMNS = [ID, GRID_ID, SECTION_ID, ECC, "HDiff1", "HDiff2"]
    """ 
    0-ID , 
    1-Grid_ID , 
    2-Section_ID ,
    3-Ecc ,
    4-HDiff1 ,
    5-HDiff2
    """

    RESULT_PERIOD_TABLE = "calEigenInf"
    RESULT_PERIOD_USEFUL_COLUMNS = [
        "ModuleID",
        "EigenNo",
        "Period",
        "Angle",
        "CoeffInf",
        "mInf",
    ]
    """
    0-moduleID
    1-EigenNo 
    2-Period Time
    3-Angle
    4-CoeffInf [unknown, xpart_ratio, ypart_ratio, zpart_ratio,
    the sum of then should be 1]
    5-MassInfo [unknown, xmass_participate, ymass_par, zmass_par,
    the sum of all xmass_par should larger than 0.9]
    """

    RESULT_MASS_TABLE = "preFlrTowProp"
    RESULT_MASS_USEFUL_COLUMNS = [FLOOR_NUM, TOWER_NUM, "MassInf"]
    """ 
    0-floor_num , 
    1-tower_num , 
    2-mass_info, list of string, [unknown, dead_load, live_load, plus_load],
    """

    RESULT_FLOOR_DATA_TABLE = "dsnStatFlrData"
    """包含了大多数楼层计算结果，包括风、地震的各类外力、承载力、刚度等等"""
    RESULT_FLOOR_DATA_USEFUL_COLUMNS_SEISMIC = [
        FLOOR_NUM,
        TOWER_NUM,
        "FlrFXInf",
        "FlrFYInf",  # 0 1 2 3
        "FlrVXInf",
        "FlrVYInf",
        "FlrMXInf",
        "FlrMYInf",  # 4 5 6 7
        "CZLXInf",
        "CZLYInf",
    ]
    """ 
    0-floor_num , 
    1-tower_num , 
    2-X方向地震外力,
    3-Y方向地震外力,
    4-X方向地震层间剪力,
    5-Y方向地震层间剪力,
    6-X方向地震倾覆力矩,
    7-Y方向地震倾覆力矩,
    8-X方向抗剪承载力,
    9-Y方向抗剪承载力,
    """

    RESULT_FLOOR_DATA_USEFUL_COLUMNS_STIFFNESS = [
        "StiffShearCutXInf",
        "StiffShearCutYInf",
        "StiffShearDisXInf",
        "StiffShearDisYInf",
        "StiffBendXInf",
        "StiffBendYInf",
    ]
    """ 
    包含了楼层的各类刚度，剪切刚度、剪力位移计算刚度、剪弯刚度
    0-剪切刚度X , 
    1-剪切刚度Y , 
    2-剪力位移计算刚度X,
    3-剪力位移计算刚度Y,
    4-剪弯刚度X,
    5-剪弯刚度Y,
    """

    RESULT_FLOOR_DATA_USEFUL_COLUMNS_WIND = [
        FLOOR_NUM,
        TOWER_NUM,
        "FlrWindFInf",
        "FlrWindVInf",  # 0 1 2 3
        "FlrWindMInf",
    ]
    """ 
    0-floor_num , 
    1-tower_num , 
    2-XY方向顺风外力,
    3-XY方向顺风剪力,
    4-XY方向顺风弯矩,
    """

    DISP_FLOOR_DATA_TABLE = "dsnStatDis"
    """包含了楼层在不同工况下的楼层位移"""

    DISP_FLOOR_DATA_USEFUL_COLUMNS_WIND = [
        FLOOR_NUM,
        TOWER_NUM,
        LOAD_CASE_ID,
        "MaxFlrAngleDis",
        "MinFlrAngleDis",
        "MaxD",
        "MassAveD",
    ]
    """
    0-floor_num , 
    1-tower_num , 
    2-loadCaseID，需要做筛选
    3-最大层间位移,
    4-最小层间位移,
    5-最大位移,
    6-平均位移,
    """

    ADJUST_COEF_TABLE = "dsnStatFlrAdjCoe"
    """楼层调整系数表格"""
    ADJUST_COEF_USEFUL_COLUMN = [FLOOR_NUM, TOWER_NUM, "JZBCoeXInf", "JZBCoeYInf"]
    """
    0-floor_num , 
    1-tower_num , 
    2-X方向剪重比调整系数
    3-Y方向剪重比调整系数
    """

    REAL_FLOOR_TABLE = "pmFlrAssembly"
    """楼层调整系数表格"""
    REAL_FLOOR_USEFUL_COLUMN = ["No", STD_FLR_ID, "Level", "Height"]
    """
    0-楼层编号 , 
    1-标准层ID , 
    2-层底高度
    3-楼层高度
    """



from typing import List


class DocTable:
    def init_content(self):
        self.context = [["None"] * self.column_num for _ in range(self.row_num)]
        self.merged_cells = set()

    def __init__(self, row_num, column_num):
        self.row_num = row_num
        self.column_num = column_num
        self.title = None
        self.no_grid = False
        self.all_bold = False
        self.init_content()

    def set_table_title(self, title):
        self.title = title

    def set_table_context(self, context):
        if len(context) == self.row_num and len(context[0]) == self.column_num:
            self.context = context
        else:
            raise ValueError("输入的内容行列数量不对")

    def merge_cells(self, i, j, k, p):
        """从第i,j个cell merge到第k,p个cell"""
        # 检查合并范围是否合法
        if i < 0 or i >= self.row_num or j < 0 or j >= self.column_num:
            raise ValueError("起始单元格超出表格范围")
        if k < 0 or k >= self.row_num or p < 0 or p >= self.column_num:
            raise ValueError("结束单元格超出表格范围")
        if i > k or (i == k and j > p):
            raise ValueError("起始单元格必须在结束单元格之前")

        # 检查是否与已合并的单元格冲突
        for start_row, start_col, end_row, end_col in self.merged_cells:
            if (i <= end_row and k > start_row) and (j <= end_col and p > start_col):
                raise ValueError("合并范围与已合并的单元格冲突")

        self.merged_cells.add((i, j, k, p))



from docx.enum.table import WD_CELL_VERTICAL_ALIGNMENT, WD_TABLE_ALIGNMENT
from docx.enum.text import WD_PARAGRAPH_ALIGNMENT, WD_BREAK
from docx.shared import Inches, RGBColor, Pt, Cm

from .BasicGenerator import BasicGenerator, PageSize
from .DocParagraph import DocParagraph
from .DocTable import DocTable
from .DocPicture import DocPicture
from .SeismicReportTemplate import SRTemplate
from .UtilFunctions import add_comma_in_num_str

from ..YDBLoader.BuildingDefine import MassResult, Period
from ..FigureGenerator.BasicPltPlotter import BasicPltPlotter, SeismicPlotter


class SeismicReportData:
    def __init__(self, name: str | None = None):
        self.project_name = name
        self.floor_num = 8
        self.yjk_version = None
        self.mass_result = None
        self.period = None
        self.__mock_data()

    def __mock_data(self):
        self.yjk_version = "6.0.0"
        self.mass_result = MassResult.mock_data()
        self.period = Period.mock_data(num=11, mass_participate=0.01)

    @property
    def is_valid(self):
        return True


class SeismicReport(BasicGenerator):
    G = 9.8
    """重力加速度"""

    def __init__(self, all_data: SeismicReportData | None = None):
        super().__init__()
        self.all_data = all_data
        # 修改为A3图纸，横向，两栏
        self.change_paper_size(PageSize.A3_LANDSCAPE, 2)
        # 修改纸张Margin，单位mm
        self.change_paper_margin(32, 25, 32, 25)
        # 格式统一修改
        self.body_style.paragraph_format.line_spacing = Pt(22)

    def creat_doc(self):
        if self.all_data == None or not self.all_data.is_valid:
            raise ValueError(
                "The data is not ready, please use set_data() to assign data."
            )
        self.__add_info()
        self.__add_seismic_chapter()

    def __add_info(self):
        model_name = self.all_data.project_name
        par_context = SRTemplate.FIRST_INFO(model_name)
        paragraph = DocParagraph(par_context)
        paragraph.style = self.body_style
        self.add_paragraph(paragraph)

    def __add_seismic_chapter(self):
        chapter_index = 8
        sub_index = 1
        self.__add_seismic_chapter_title(chapter_index)
        sub_index = self.__add_seismic_embedding(chapter_index, sub_index)
        sub_index = self.__add_project_mass(chapter_index, sub_index)
        sub_index = self.__add_period(chapter_index, sub_index)
        sub_index = self.__add_shear_mass_ratio(chapter_index, sub_index)
        sub_index = self.__add_shear_and_moment(chapter_index, sub_index)
        sub_index = self.__add_horizental_moment_ratio_for_column(
            chapter_index, sub_index
        )
        sub_index = self.__add_disp_and_drift(chapter_index, sub_index)
        sub_index = self.__add_horizental_stiffness_ratio(chapter_index, sub_index)
        sub_index = self.__add_rotation_ratio(chapter_index, sub_index)
        sub_index = self.__add_stiffness_mass_ratio(chapter_index, sub_index)
        sub_index = self.__add_shear_capacity_ratio(chapter_index, sub_index)
        sub_index = self.__add_wind_acc(chapter_index, sub_index)

    def __add_seismic_chapter_title(self, chapter_index: int):
        # 获取需要的数据
        yjk_version = self.all_data.yjk_version
        # 开始生成报告
        current_context = SRTemplate.SEISMIC_CHAPTER_TITLE
        par_context = DocParagraph(current_context.title(chapter_index))
        par_context.par_level = 1
        self.add_title(par_context, 12, 6)
        paragraph_texts = current_context.paragraph(chapter_index, yjk_version)
        for context in paragraph_texts[:-1]:
            paragraph = DocParagraph(context)
            paragraph.style = self.body_style
            self.add_paragraph(paragraph)

        text = paragraph_texts[-1]
        paragraph = DocParagraph(text)
        paragraph.style = self.body_style
        paragraph.alignment = WD_PARAGRAPH_ALIGNMENT.CENTER
        paragraph.first_line_indent = 0
        self.add_paragraph(paragraph)

        figure_title = current_context.picture(chapter_index)
        self.__insert_table_figure_title(figure_title)

    def __add_seismic_embedding(self, chapter_index: int, sub_index: int):

        current_context = SRTemplate.SEISMIC_EMBEDDING
        self.__insert_title_par_2(current_context, chapter_index, sub_index)

        context = current_context.paragraph(chapter_index, sub_index)[0]
        self.__insert_normal_para(context)

        table_title = current_context.table(chapter_index, sub_index)
        self.__insert_table_figure_title(table_title)

        table = DocTable(3, 7)
        table.merge_cells(1, 4, 2, 4)
        table.merge_cells(1, 5, 2, 5)
        table.set_table_context(current_context.table_context)
        self.add_table(table)

        return sub_index + 1

    def __add_project_mass(self, chapter_index: int, sub_index: int):
        mass_result = self.all_data.mass_result
        dead_load = mass_result.total_dead_load
        live_load = mass_result.total_live_load * 0.5
        total_load = mass_result.total_load
        total_area = mass_result.total_slab_area
        average_dead_load = f"{dead_load / total_area:.1f}"
        average_live_load = f"{live_load / total_area:.1f}"
        average_total_load = f"{total_load / total_area:.1f}"

        current_context = SRTemplate.PROJECT_MASS
        self.__insert_title_par_2(current_context, chapter_index, sub_index)

        contexts = current_context.paragraph(
            chapter_index,
            sub_index,
            total_mass=add_comma_in_num_str(int(total_load / SeismicReport.G)),
            total_area=add_comma_in_num_str(int(total_area)),
            average_load=average_total_load,
        )
        paragraph = DocParagraph(contexts[0])
        paragraph.style = self.body_style
        self.add_paragraph(paragraph)

        table_title = current_context.table(chapter_index, sub_index)
        self.__insert_table_figure_title(table_title)

        table = DocTable(4, 4)
        table.set_table_context(
            current_context.table_context(
                dead_mass=add_comma_in_num_str(int(dead_load / SeismicReport.G)),
                live_mass=add_comma_in_num_str(int(live_load / SeismicReport.G)),
                total_mass=add_comma_in_num_str(int(total_load / SeismicReport.G)),
                dead_percentage=f"{dead_load/total_load*100:.1f}%",
                live_percentage=f"{live_load/total_load*100:.1f}%",
                total_percentage="100%",
                dead_average=average_dead_load,
                live_average=average_live_load,
                total_average=average_total_load,
            )
        )
        self.add_table(table)

        return sub_index + 1

    def __add_period(self, chapter_index: int, sub_index: int):
        current_context = SRTemplate.PERIOD
        self.__insert_title_par_2(current_context, chapter_index, sub_index)

        paras = current_context.paragraph(
            self.all_data.period, chapter_index, sub_index
        )
        self.__insert_normal_para(paras[0])

        table_title = current_context.table(chapter_index, sub_index)
        self.__insert_table_figure_title(table_title)
        period_num = len(self.all_data.period.periods)
        if period_num <= 10:
            row_num = period_num + 1
        else:
            row_num = 12
        table = DocTable(row_num, 6)
        table_context = current_context.table_context
        last_mass_participate_x = 0
        last_mass_participate_y = 0
        for i in range(row_num - 1):
            temp_period = self.all_data.period.periods[i if i <= 9 else row_num - 2]
            if i <= 8 or (i == 9 and row_num == 11) or i > 9:
                table_context.append(
                    [
                        str(i + 1),
                        temp_period.time_str,
                        temp_period.movement_coeff,
                        temp_period.rotation_coeff,
                        temp_period.get_mass_participate_x(last_mass_participate_x),
                        temp_period.get_mass_participate_y(last_mass_participate_y),
                    ]
                )
            elif i == 9:
                table_context.append(["..."] * 6)
            last_mass_participate_x += temp_period.mass_participate_x
            last_mass_participate_y += temp_period.mass_participate_y
        table.set_table_context(table_context)
        self.add_table(table)

        self.__insert_normal_para(paras[1])

        text = "*{（这里需要振型图片！）}"
        titles = ["(a) 第一振型", "(b) 第二振型", "(c) 第三振型"]
        for i in range(3):
            paragraph = DocParagraph(text)
            paragraph.style = self.body_style
            paragraph.alignment = WD_PARAGRAPH_ALIGNMENT.CENTER
            paragraph.first_line_indent = 0
            self.add_paragraph(paragraph)
            self.__insert_table_figure_title(titles[i])
        figure_title = current_context.picture(chapter_index, sub_index)
        self.__insert_table_figure_title(figure_title)

        return sub_index + 1

    def __add_shear_mass_ratio(self, chapter_index: int, sub_index: int):
        current_context = SRTemplate.SHEAR_MASS_RATIO
        self.__insert_title_par_2(current_context, chapter_index, sub_index)

        para = current_context.paragraph(chapter_index, sub_index)
        self.__insert_normal_para(para)

        table_title = current_context.table(chapter_index, sub_index)
        self.__insert_table_figure_title(table_title)
        table = DocTable(3, 4)
        table.merge_cells(1, 2, 2, 2)
        table.set_table_context(current_context.table_context())
        self.add_table(table)

        figure = SeismicPlotter()
        figure.test_plot()
        stream = figure.save_to_stream()
        picture = DocPicture(stream, 2)
        self.add_picture(picture)

        figure_title = current_context.picture(chapter_index, sub_index)
        self.__insert_table_figure_title(figure_title)

        return sub_index + 1

    def __add_shear_and_moment(self, chapter_index: int, sub_index: int):
        current_context = SRTemplate.SHEAR_AND_MOMENT
        self.__insert_title_par_2(current_context, chapter_index, sub_index)

        para = current_context.paragraph(chapter_index, sub_index)
        self.__insert_normal_para(para)

        table_title = current_context.table(chapter_index, sub_index)
        self.__insert_table_figure_title(table_title)
        table = DocTable(6, 4)
        table.merge_cells(0, 0, 1, 1)
        table.merge_cells(0, 2, 0, 3)
        table.merge_cells(2, 0, 3, 0)
        table.merge_cells(4, 0, 5, 0)
        table.set_table_context(current_context.table_context())
        self.add_table(table)

        figure = SeismicPlotter()
        figure.test_plot()
        stream = figure.save_to_stream()
        picture = DocPicture(stream, 2)
        self.add_picture(picture)

        figure_titles = current_context.picture(chapter_index, sub_index)
        self.__insert_table_figure_title(figure_titles[0])

        figure = SeismicPlotter()
        figure.test_plot()
        stream = figure.save_to_stream()
        picture = DocPicture(stream, 2)
        self.add_picture(picture)

        self.__insert_table_figure_title(figure_titles[1])

        return sub_index + 1

    def __add_horizental_moment_ratio_for_column(
        self, chapter_index: int, sub_index: int
    ):
        current_context = SRTemplate.HORIZENTAL_MOMENT_RATIO_FOR_COLUMN
        self.__insert_title_par_2(current_context, chapter_index, sub_index)

        paras = current_context.paragraph(chapter_index, sub_index)
        self.__insert_normal_para(paras[0])
        self.__insert_normal_para(paras[1])

        figure = SeismicPlotter()
        figure.test_plot()
        stream = figure.save_to_stream()
        picture = DocPicture(stream, 2)
        self.add_picture(picture)

        figure_title = current_context.picture(chapter_index, sub_index)
        self.__insert_table_figure_title(figure_title)

        table_titles = current_context.table(chapter_index, sub_index)
        self.__insert_table_figure_title(table_titles[0])
        row_num = self.all_data.floor_num + 1
        table = DocTable(row_num, 5)
        table_context = current_context.table_context()
        for _ in range(self.all_data.floor_num):
            table_context.append(["--"] * 5)
        table.set_table_context(table_context)
        self.add_table(table)

        self.__insert_table_figure_title(table_titles[1])
        row_num = self.all_data.floor_num + 1
        table = DocTable(row_num, 5)
        table_context = current_context.table_context()
        for _ in range(self.all_data.floor_num):
            table_context.append(["--"] * 5)
        table.set_table_context(table_context)
        self.add_table(table)

        return sub_index + 1

    def __add_disp_and_drift(self, chapter_index: int, sub_index: int):
        current_context = SRTemplate.DISP_AND_DRIFT
        self.__insert_title_par_2(current_context, chapter_index, sub_index)

        return sub_index + 1

    def __add_horizental_stiffness_ratio(self, chapter_index: int, sub_index: int):
        current_context = SRTemplate.HORIZENTAL_STIFFNESS_RATIO
        self.__insert_title_par_2(current_context, chapter_index, sub_index)

        return sub_index + 1

    def __add_rotation_ratio(self, chapter_index: int, sub_index: int):
        current_context = SRTemplate.ROTATION_RATIO
        self.__insert_title_par_2(current_context, chapter_index, sub_index)

        return sub_index + 1

    def __add_stiffness_mass_ratio(self, chapter_index: int, sub_index: int):
        current_context = SRTemplate.STIFFNESS_MASS_RATIO
        self.__insert_title_par_2(current_context, chapter_index, sub_index)

        return sub_index + 1

    def __add_shear_capacity_ratio(self, chapter_index: int, sub_index: int):
        current_context = SRTemplate.SHEAR_CAPACITY_RATIO
        self.__insert_title_par_2(current_context, chapter_index, sub_index)

        return sub_index + 1

    def __add_wind_acc(self, chapter_index: int, sub_index: int):
        current_context = SRTemplate.WIND_ACC
        self.__insert_title_par_2(current_context, chapter_index, sub_index)

        return sub_index + 1

    def __insert_title_par_2(self, current_context, chapter_index, sub_index):
        """用于生成二级子目录"""
        par_context = DocParagraph(current_context.title(chapter_index, sub_index))
        par_context.par_level = 2
        self.add_title(par_context, 6, 6)

    def __insert_normal_para(self, context):
        paragraph = DocParagraph(context)
        paragraph.style = self.body_style
        self.add_paragraph(paragraph)

    def __insert_table_figure_title(self, context):
        paragraph = DocParagraph(context)
        paragraph.style = self.small_title_style
        paragraph.alignment = WD_PARAGRAPH_ALIGNMENT.CENTER
        self.add_paragraph(paragraph)



from ..BasicPltPlotter import SeismicPlotter, GetTicks
from typing import List
import numpy as np
import matplotlib.pyplot as plt


class ShearMassRatioPlotter(SeismicPlotter):
    def __init__(self, fig_num=2, floor_num=8):
        super().__init__(fig_num, floor_num)
        self.__limit = None
        self.type = "剪重比"

    def set_data(self, shear_x: List[float], shear_y: List[float], mass: List[float]):
        if len(shear_x) != self.floor_num:
            raise ValueError(
                f"Lenght of shear_x is not equal to floor number: {self.floor_num}!"
            )
        if len(shear_y) != self.floor_num:
            raise ValueError(
                f"Lenght of shear_y is not equal to floor number: {self.floor_num}!"
            )
        if len(mass) != self.floor_num:
            raise ValueError(
                f"Lenght of mass is not equal to floor number: {self.floor_num}!"
            )

        self._ax1_x = np.array(shear_x) / np.array(mass)
        self._ax2_x = np.array(shear_y) / np.array(mass)

    def set_limit(self, limit: float):
        self.__limit = limit

    def plot(self):
        if self.__limit:
            self.__plot_limit()
        kwargs_x = self.kwargs_x.copy()
        kwargs_x["label"] = "X"
        kwargs_y = self.kwargs_x.copy()
        kwargs_y["label"] = "Y"
        self.axes[0].plot(self._ax1_x, self._y_values, **kwargs_x)
        self.axes[1].plot(self._ax2_x, self._y_values, **kwargs_y)
        self.__adjust_lim()
        self.__add_titles()

    def __plot_limit(self):
        limitation = self.__limit
        for ax in self.axes:
            ax.vlines(
                x=limitation,
                ymin=0,
                ymax=self.floor_num,
                color="r",
                linewidth=3,
                ls="--",
                label=f"限值{limitation*100:.1f}%",
            )

    def __adjust_lim(self):
        xmaxs = [self._ax1_x.max(), self._ax2_x.max()]
        for i in range(2):
            self.axes[i].set_xlim(left=0, right=xmaxs[i] * 1.2)
            self.axes[i].set_yticks(self._y_major_ticks)
            self.axes[i].set_yticks(self._y_minor_ticks, minor=True)
            x_ticks = GetTicks(xmaxs[i])
            self.axes[i].set_xticks(x_ticks)
            self.axes[i].set_xticklabels([f"{i*100:.1f}%" for i in x_ticks])

    def __add_titles(self):
        self.axes[0].set_ylabel(self.y_label)
        self.axes[0].set_xlabel(f"X小震下{self.type}")
        self.axes[1].set_xlabel(f"Y小震下{self.type}")
        self.axes[0].legend(framealpha=0, fontsize=12, loc=4)
        self.axes[1].legend(framealpha=0, fontsize=12, loc=4)



from enum import Enum


class LoadParams:
    def __init__(self):
        self.append_dead_load = 1
        self.live_load = 3.5
        self.dead_load_coef = 1.3
        self.live_load_coef = 1.5
        self.live_load_adjust_coef = 1
        self.live_load_permenent_coef = 0.5


class LoadType(Enum):
    Dead = 1
    Live = 2


class LoadCalulateType(Enum):
    qk = 1
    q = 2
    qe = 3


class StairLoad:
    def __init__(self, dead, live, load_params: LoadParams):
        self.dead = dead
        self.live = live
        self.load_params = load_params

    @property
    def qk(self):
        return self.dead + self.live

    @property
    def q(self):
        return (
            self.load_params.dead_load_coef * self.dead
            + self.load_params.live_load_coef * self.live
        )

    @property
    def qe(self):
        return self.dead + self.load_params.live_load_permenent_coef * self.live



from ..BasicPltPlotter import SeismicPlotter, GetTicks
from typing import List
import numpy as np
import matplotlib.pyplot as plt


class DriftPlotter(SeismicPlotter):
    def __init__(self, floor_num=8, fig_num=2):
        super().__init__(fig_num, floor_num)
        self.__limit = None
        self.type = "层间位移角"

    def set_data(
        self,
        wind_x: List[float],
        wind_y: List[float],
        seismic_x: List[float],
        seismic_y: List[float],
    ):
        # 验证数据长度
        data_dict = {
            "wind_x": wind_x,
            "wind_y": wind_y,
            "seismic_x": seismic_x,
            "seismic_y": seismic_y,
        }
        for name, data_list in data_dict.items():
            self._validate_list_length(data_list, name)

        self._ax1_x = 1 / np.array(wind_x)
        self._ax1_y = 1 / np.array(wind_y)
        self._ax2_x = 1 / np.array(seismic_x)
        self._ax2_y = 1 / np.array(seismic_y)

    def plot(self):
        if self.__limit:
            self.__plot_limit()
        kwargs_x = self.kwargs_x.copy()
        kwargs_x["label"] = "X风"
        kwargs_y = self.kwargs_y.copy()
        kwargs_y["label"] = "Y风"
        self.axes[0].plot(self._ax1_x, self._y_values, **kwargs_x)
        self.axes[0].plot(self._ax1_y, self._y_values, **kwargs_y)
        kwargs_x["label"] = "X小震"
        kwargs_y["label"] = "Y小震"
        self.axes[1].plot(self._ax2_x, self._y_values, **kwargs_x)
        self.axes[1].plot(self._ax2_y, self._y_values, **kwargs_y)
        self.__adjust_lim()
        self.__add_titles()

    def __adjust_lim(self):
        xmaxs = [self._ax1_x.max(), self._ax2_x.max()]
        for i in range(2):
            self.axes[i].set_xlim(left=0, right=xmaxs[i] * 1.2)
            self.axes[i].set_yticks(self._y_major_ticks)
            self.axes[i].set_yticks(self._y_minor_ticks, minor=True)
            x_ticks = GetTicks(xmaxs[i])
            self.axes[i].set_xticks(x_ticks)
            # self.axes[i].set_xticklabels([f"{i*100:.1f}%" for i in x_ticks])

    def __add_titles(self):
        self.axes[0].set_ylabel(self.y_label)
        self.axes[0].set_xlabel(f"风下{self.type}")
        self.axes[1].set_xlabel(f"小震下{self.type}")
        self.axes[0].legend(framealpha=0, fontsize=12, loc=1)
        self.axes[1].legend(framealpha=0, fontsize=12, loc=1)



import math


class Joint:
    def __init__(self, id: int, x: float, y: float, std_flr_id: int):
        self.id = id
        self.x = x
        self.y = y
        self.std_flr_id = std_flr_id

    def distance_to_joint(self, other) -> float:
        if not isinstance(other, Joint):
            raise ValueError("Input should be Joint")
        dis_square = (self.x - other.x) ** 2 + (self.y - other.y) ** 2
        return math.sqrt(dis_square)

    def __str__(self):
        return f"Joint(id:{self.id}):[x:{self.x:.4f},y:{self.y:.4f}]:stdFlrId:{self.std_flr_id}"

    def __repr__(self):
        return str(self)


if __name__ == "__main__":
    j = Joint(1, 3.1, 36.6513246534313)
    print(j)



from setuptools import setup, find_packages

setup(
    name="civil_tools_v",  # 包的名称
    version="0.0.4",  # 包的版本号
    author="Xinyu Gao (Vincent)",  # 作者姓名
    author_email="just_gxy@163.com",  # 作者邮箱
    description="A powerful tool for civil engineer in their work.",  # 包的简短描述
    long_description=open("README.md", encoding="utf-8").read(),  # 包的详细描述
    long_description_content_type="text/markdown",  # 详细描述的内容类型
    url="https://github.com/VincentXGao/civil-tools",  # 项目的 URL
    packages=find_packages(exclude=["tests"]),  # 自动发现并包含所有的包
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],  # 包的分类信息
    install_requires=[
        "numpy>=2.0",
        "python-docx>=1.0",
        "matplotlib>=3.10.0",
        "ezdxf==1.4.0",
    ],  # 依赖的库及其最低版本
    python_requires=">=3.6",  # 所需的 Python 版本
)



from ..Section import Section, ShapeEnum
from ..Geometry import Joint


class Beam:
    def __init__(self, id: int, start_joint: Joint, end_joint: Joint, section: Section):
        self.id = id
        self.start_joint = start_joint
        self.end_joint = end_joint
        self.section = section
        self.std_flr_id = self.start_joint.std_flr_id

    def get_data_for_plot(self):
        return [
            [self.start_joint.x, self.end_joint.x],
            [self.start_joint.y, self.end_joint.y],
        ]

    def __str__(self):
        return f"Beam:{self.id}-{str(self.section)}"

    def __repr__(self):
        return self.__str__()
